summaryrefslogtreecommitdiffhomepage
diff options
context:
space:
mode:
authorMojgan <mojgan.jelodar@mullvad.net>2025-12-05 11:26:22 +0100
committerMojgan <mojgan.jelodar@mullvad.net>2025-12-05 11:26:22 +0100
commite505a7590c21ebd1421cb5073d103e389d26cd41 (patch)
treec6f5351949c3b7187c268b7106d9f2d4c1270b95
parent71f1be8cc1b39815e816cd37a75e0659a1497c35 (diff)
downloadmullvadvpn-bug-bash-location-view.tar.xz
mullvadvpn-bug-bash-location-view.zip
Add recent connectionsbug-bash-location-view
-rw-r--r--ios/Assets/Localizable.xcstrings17
-rw-r--r--ios/CHANGELOG.md1
-rw-r--r--ios/MullvadSettings/RecentConnections.swift16
-rw-r--r--ios/MullvadSettings/RecentConnectionsRepository.swift97
-rw-r--r--ios/MullvadSettings/RecentConnectionsRepositoryProtocol.swift15
-rw-r--r--ios/MullvadTypes/RelayLocation.swift1
-rw-r--r--ios/MullvadVPN.xcodeproj/project.pbxproj38
-rw-r--r--ios/MullvadVPN/AppDelegate.swift11
-rw-r--r--ios/MullvadVPN/Classes/AccessbilityIdentifier.swift2
-rw-r--r--ios/MullvadVPN/Coordinators/ApplicationCoordinator.swift3
-rw-r--r--ios/MullvadVPN/Coordinators/CustomLists/AddLocationsDataSource.swift6
-rw-r--r--ios/MullvadVPN/Coordinators/LocationCoordinator.swift8
-rw-r--r--ios/MullvadVPN/View controllers/AccountDeletion/AccountDeletionView.swift2
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/AllLocationDataSource.swift8
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/CustomListsDataSource.swift25
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationDataSourceProtocol.swift96
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNode.swift68
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNodeResolver.swift73
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentListDataSource.swift48
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentsInteractor.swift110
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/LocationContext.swift5
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/LocationSection.swift21
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/MockSelectLocationViewModel.swift16
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/SelectLocationViewModel.swift181
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/EntryLocationView.swift4
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/ExitLocationView.swift48
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/Hop.swift15
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/MultihopSelectionView.swift4
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationListItem.swift32
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationsListView.swift57
-rw-r--r--ios/MullvadVPN/View controllers/SelectLocation/Views/SelectLocationView.swift105
-rw-r--r--ios/MullvadVPN/Views/MullvadListSectionFooter.swift23
-rw-r--r--ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/AllLocationsDataSourceTests.swift135
-rw-r--r--ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/CustomListsDataSourceTests.swift32
-rw-r--r--ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/LocationNodeResolverTests.swift75
-rw-r--r--ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/RecentConnectionsRepositoryTests.swift129
36 files changed, 1080 insertions, 447 deletions
diff --git a/ios/Assets/Localizable.xcstrings b/ios/Assets/Localizable.xcstrings
index dbfb944e84..3ebc5700e0 100644
--- a/ios/Assets/Localizable.xcstrings
+++ b/ios/Assets/Localizable.xcstrings
@@ -957,6 +957,9 @@
}
}
},
+ "%@ recents" : {
+
+ },
"%@ via %@" : {
"localizations" : {
"da" : {
@@ -16679,6 +16682,9 @@
}
}
},
+ "Disable" : {
+
+ },
"Disable all \"%@\" above to activate this setting." : {
"localizations" : {
"da" : {
@@ -16915,6 +16921,9 @@
}
}
},
+ "Disabling recents will also clear history." : {
+
+ },
"Discard changes" : {
"localizations" : {
"da" : {
@@ -34998,6 +35007,7 @@
}
},
"No matching relays found, check your filter settings." : {
+ "extractionState" : "stale",
"localizations" : {
"da" : {
"stringUnit" : {
@@ -35469,6 +35479,9 @@
}
}
},
+ "No recent selection history" : {
+
+ },
"No result for \"%@\", please try a different search term." : {
},
@@ -41263,6 +41276,9 @@
}
}
},
+ "Recents" : {
+
+ },
"RECONNECTING" : {
"localizations" : {
"da" : {
@@ -52259,6 +52275,7 @@
},
"To create a custom list, tap on \"...\" " : {
+ "extractionState" : "stale",
"localizations" : {
"da" : {
"stringUnit" : {
diff --git a/ios/CHANGELOG.md b/ios/CHANGELOG.md
index 74aa8de4df..1003455a77 100644
--- a/ios/CHANGELOG.md
+++ b/ios/CHANGELOG.md
@@ -24,6 +24,7 @@ Line wrap the file at 100 chars. Th
## UNRELEASED
### Added
- Add support for additional languages.
+- Add recent connections in the Select Location view.
### Added
- Allow using port 443 for UDP-over-TCP.
diff --git a/ios/MullvadSettings/RecentConnections.swift b/ios/MullvadSettings/RecentConnections.swift
index d4a83c348b..0a0fcc602c 100644
--- a/ios/MullvadSettings/RecentConnections.swift
+++ b/ios/MullvadSettings/RecentConnections.swift
@@ -8,7 +8,17 @@
import MullvadTypes
public struct RecentConnections: Codable, Sendable, Equatable {
- let isEnabled: Bool
- let entryLocations: [UserSelectedRelays]
- let exitLocations: [UserSelectedRelays]
+ public let isEnabled: Bool
+ public let entryLocations: [UserSelectedRelays]
+ public let exitLocations: [UserSelectedRelays]
+
+ public init(
+ isEnabled: Bool = true,
+ entryLocations: [UserSelectedRelays] = [],
+ exitLocations: [UserSelectedRelays] = []
+ ) {
+ self.isEnabled = isEnabled
+ self.entryLocations = entryLocations
+ self.exitLocations = exitLocations
+ }
}
diff --git a/ios/MullvadSettings/RecentConnectionsRepositoryProtocol.swift b/ios/MullvadSettings/RecentConnectionsRepositoryProtocol.swift
index a7521e6f56..b62432b63a 100644
--- a/ios/MullvadSettings/RecentConnectionsRepositoryProtocol.swift
+++ b/ios/MullvadSettings/RecentConnectionsRepositoryProtocol.swift
@@ -5,13 +5,18 @@
// Created by Mojgan on 2025-10-15.
// Copyright © 2025 Mullvad VPN AB. All rights reserved.
//
+
+import Combine
import MullvadTypes
-public enum RecentLocationType: CaseIterable {
- case entry, exit
+public enum RecentConnectionsResult {
+ case success(RecentConnections)
+ case failure(Error)
}
public protocol RecentConnectionsRepositoryProtocol {
- func setRecentsEnabled(_ isEnabled: Bool) throws
- func add(_ location: UserSelectedRelays, as: RecentLocationType) throws
- func all() throws -> RecentConnections
+ var recentConnectionsPublisher: AnyPublisher<RecentConnectionsResult, Never> { get }
+ func disable()
+ func enable(_ selectedEntryRelays: UserSelectedRelays?, selectedExitRelays: UserSelectedRelays)
+ func add(_ selectedEntryRelays: UserSelectedRelays?, selectedExitRelays: UserSelectedRelays)
+ func initiate()
}
diff --git a/ios/MullvadTypes/RelayLocation.swift b/ios/MullvadTypes/RelayLocation.swift
index a739c8aa09..2cb2b583a0 100644
--- a/ios/MullvadTypes/RelayLocation.swift
+++ b/ios/MullvadTypes/RelayLocation.swift
@@ -118,6 +118,7 @@ public struct UserSelectedRelays: Codable, Equatable, Sendable {
}
extension UserSelectedRelays {
+ public static let `default`: UserSelectedRelays = UserSelectedRelays(locations: [.country("se")])
public struct CustomListSelection: Codable, Equatable, Sendable {
/// The ID of the custom list that the selected relays belong to.
public let listId: UUID
diff --git a/ios/MullvadVPN.xcodeproj/project.pbxproj b/ios/MullvadVPN.xcodeproj/project.pbxproj
index 273bf1ef64..78b1b244e2 100644
--- a/ios/MullvadVPN.xcodeproj/project.pbxproj
+++ b/ios/MullvadVPN.xcodeproj/project.pbxproj
@@ -899,9 +899,16 @@
F0164ED12B4F2DCB0020268D /* AccessMethodIterator.swift in Sources */ = {isa = PBXBuildFile; fileRef = F0164ED02B4F2DCB0020268D /* AccessMethodIterator.swift */; };
F017F8E02D78AC020076EC01 /* RelayFilterDataSourceItem.swift in Sources */ = {isa = PBXBuildFile; fileRef = F017F8DF2D78ABE90076EC01 /* RelayFilterDataSourceItem.swift */; };
F01DAE332C2B032A00521E46 /* RelaySelection.swift in Sources */ = {isa = PBXBuildFile; fileRef = F01DAE322C2B032A00521E46 /* RelaySelection.swift */; };
+ F01F38222EE1DE3400A4DC17 /* LocationNodeResolverTests.swift in Sources */ = {isa = PBXBuildFile; fileRef = F01F38212EE1DE2D00A4DC17 /* LocationNodeResolverTests.swift */; };
F022EBA62CF0C6AE009484B9 /* ConsolidatedApplicationLog.swift in Sources */ = {isa = PBXBuildFile; fileRef = 5871FB95254ADE4E0051A0A4 /* ConsolidatedApplicationLog.swift */; };
+ F02844E22ECF5DB2007EC8F2 /* LocationNodeResolver.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02844E12ECF5D9A007EC8F2 /* LocationNodeResolver.swift */; };
+ F02844E32ECF5DB2007EC8F2 /* LocationNodeResolver.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02844E12ECF5D9A007EC8F2 /* LocationNodeResolver.swift */; };
F028A56A2A34D4E700C0CAA3 /* RedeemVoucherViewController.swift in Sources */ = {isa = PBXBuildFile; fileRef = F028A5692A34D4E700C0CAA3 /* RedeemVoucherViewController.swift */; };
F028A56C2A34D8E600C0CAA3 /* AddCreditSucceededViewController.swift in Sources */ = {isa = PBXBuildFile; fileRef = F028A56B2A34D8E600C0CAA3 /* AddCreditSucceededViewController.swift */; };
+ F02BFFC92ED5BBF4007CEA69 /* RecentsInteractor.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02BFFC72ED5BBEB007CEA69 /* RecentsInteractor.swift */; };
+ F02BFFCB2ED5D3CB007CEA69 /* MullvadListSectionFooter.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02BFFCA2ED5D3C9007CEA69 /* MullvadListSectionFooter.swift */; };
+ F02BFFCD2ED89886007CEA69 /* RecentLocationListItem.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02BFFCC2ED89872007CEA69 /* RecentLocationListItem.swift */; };
+ F02BFFCF2ED89996007CEA69 /* RecentLocationsListView.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02BFFCE2ED8997D007CEA69 /* RecentLocationsListView.swift */; };
F02F41A02B9723AF00625A4F /* AddLocationsViewController.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02F419A2B9723AE00625A4F /* AddLocationsViewController.swift */; };
F02F41A12B9723AF00625A4F /* AddLocationsDataSource.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02F419B2B9723AE00625A4F /* AddLocationsDataSource.swift */; };
F02F41A22B9723AF00625A4F /* AddLocationsCoordinator.swift in Sources */ = {isa = PBXBuildFile; fileRef = F02F419C2B9723AF00625A4F /* AddLocationsCoordinator.swift */; };
@@ -961,6 +968,8 @@
F08B6B7D2C528C6300D0A121 /* EphemeralPeerExchangingPipeline.swift in Sources */ = {isa = PBXBuildFile; fileRef = F05919762C453FAF00C301F3 /* EphemeralPeerExchangingPipeline.swift */; };
F08B6B7E2C528C6300D0A121 /* MultiHopEphemeralPeerExchanger.swift in Sources */ = {isa = PBXBuildFile; fileRef = F059197C2C454C9200C301F3 /* MultiHopEphemeralPeerExchanger.swift */; };
F09084682C6E88ED001CD36E /* DaitaPromptAlert.swift in Sources */ = {isa = PBXBuildFile; fileRef = F09084672C6E88ED001CD36E /* DaitaPromptAlert.swift */; };
+ F09444132ECCA4B30059606A /* RecentListDataSource.swift in Sources */ = {isa = PBXBuildFile; fileRef = F09444122ECCA4990059606A /* RecentListDataSource.swift */; };
+ F09444142ECCA4B30059606A /* RecentListDataSource.swift in Sources */ = {isa = PBXBuildFile; fileRef = F09444122ECCA4990059606A /* RecentListDataSource.swift */; };
F09A297B2A9F8A9B00EA3B6F /* LogoutDialogueView.swift in Sources */ = {isa = PBXBuildFile; fileRef = F09A29782A9F8A9B00EA3B6F /* LogoutDialogueView.swift */; };
F09A297C2A9F8A9B00EA3B6F /* VoucherTextField.swift in Sources */ = {isa = PBXBuildFile; fileRef = F09A29792A9F8A9B00EA3B6F /* VoucherTextField.swift */; };
F09A297D2A9F8A9B00EA3B6F /* RedeemVoucherContentView.swift in Sources */ = {isa = PBXBuildFile; fileRef = F09A297A2A9F8A9B00EA3B6F /* RedeemVoucherContentView.swift */; };
@@ -2343,10 +2352,16 @@
F01765962E4A324F00262F54 /* fi */ = {isa = PBXFileReference; lastKnownFileType = text.plist.strings; name = fi; path = fi.lproj/InfoPlist.strings; sourceTree = "<group>"; };
F017F8DF2D78ABE90076EC01 /* RelayFilterDataSourceItem.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = RelayFilterDataSourceItem.swift; sourceTree = "<group>"; };
F01DAE322C2B032A00521E46 /* RelaySelection.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = RelaySelection.swift; sourceTree = "<group>"; };
+ F01F38212EE1DE2D00A4DC17 /* LocationNodeResolverTests.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = LocationNodeResolverTests.swift; sourceTree = "<group>"; };
F027E0982E4A325C00090D26 /* it */ = {isa = PBXFileReference; lastKnownFileType = text.plist.strings; name = it; path = it.lproj/InfoPlist.strings; sourceTree = "<group>"; };
+ F02844E12ECF5D9A007EC8F2 /* LocationNodeResolver.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = LocationNodeResolver.swift; sourceTree = "<group>"; };
F028A5692A34D4E700C0CAA3 /* RedeemVoucherViewController.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = RedeemVoucherViewController.swift; sourceTree = "<group>"; };
F028A56B2A34D8E600C0CAA3 /* AddCreditSucceededViewController.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = AddCreditSucceededViewController.swift; sourceTree = "<group>"; };
F029DF502E4A328B00143913 /* sv */ = {isa = PBXFileReference; lastKnownFileType = text.plist.strings; name = sv; path = sv.lproj/InfoPlist.strings; sourceTree = "<group>"; };
+ F02BFFC72ED5BBEB007CEA69 /* RecentsInteractor.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = RecentsInteractor.swift; sourceTree = "<group>"; };
+ F02BFFCA2ED5D3C9007CEA69 /* MullvadListSectionFooter.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = MullvadListSectionFooter.swift; sourceTree = "<group>"; };
+ F02BFFCC2ED89872007CEA69 /* RecentLocationListItem.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = RecentLocationListItem.swift; sourceTree = "<group>"; };
+ F02BFFCE2ED8997D007CEA69 /* RecentLocationsListView.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = RecentLocationsListView.swift; sourceTree = "<group>"; };
F02F419A2B9723AE00625A4F /* AddLocationsViewController.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = AddLocationsViewController.swift; sourceTree = "<group>"; };
F02F419B2B9723AE00625A4F /* AddLocationsDataSource.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = AddLocationsDataSource.swift; sourceTree = "<group>"; };
F02F419C2B9723AF00625A4F /* AddLocationsCoordinator.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = AddLocationsCoordinator.swift; sourceTree = "<group>"; };
@@ -2397,6 +2412,7 @@
F07F1BD42E4A361E003FB336 /* zh-Hant */ = {isa = PBXFileReference; lastKnownFileType = text.plist.strings; name = "zh-Hant"; path = "zh-Hant.lproj/InfoPlist.strings"; sourceTree = "<group>"; };
F08096FA2E4A325600D8B0CF /* fr */ = {isa = PBXFileReference; lastKnownFileType = text.plist.strings; name = fr; path = fr.lproj/InfoPlist.strings; sourceTree = "<group>"; };
F09084672C6E88ED001CD36E /* DaitaPromptAlert.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = DaitaPromptAlert.swift; sourceTree = "<group>"; };
+ F09444122ECCA4990059606A /* RecentListDataSource.swift */ = {isa = PBXFileReference; lastKnownFileType = sourcecode.swift; path = RecentListDataSource.swift; sourceTree = "<group>"; };
F09A29782A9F8A9B00EA3B6F /* LogoutDialogueView.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = LogoutDialogueView.swift; sourceTree = "<group>"; };
F09A29792A9F8A9B00EA3B6F /* VoucherTextField.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = VoucherTextField.swift; sourceTree = "<group>"; };
F09A297A2A9F8A9B00EA3B6F /* RedeemVoucherContentView.swift */ = {isa = PBXFileReference; fileEncoding = 4; lastKnownFileType = sourcecode.swift; path = RedeemVoucherContentView.swift; sourceTree = "<group>"; };
@@ -3312,6 +3328,7 @@
7A0EAE992D01B41500D3EB8B /* MainButtonStyle.swift */,
F97C38E72DF025D9006DCB08 /* MullvadAlert.swift */,
F91B94A62DC9EB5E00132C28 /* MullvadInfoHeaderView.swift */,
+ F02BFFCA2ED5D3C9007CEA69 /* MullvadListSectionFooter.swift */,
F96D04EA2EC317EC004A4D48 /* MullvadListSectionHeader.swift */,
F90A988B2E1268510020F64F /* MullvadPrimaryTextField.swift */,
F91CCBFB2DFAF5E1007F1925 /* MullvadProgressViewStyle.swift */,
@@ -4365,6 +4382,7 @@
7A9BE5AC2B90DF2D00E2A7D0 /* AllLocationsDataSourceTests.swift */,
F050AE5B2B73797D003F4EDB /* CustomListRepositoryTests.swift */,
7A9BE5A42B90760C00E2A7D0 /* CustomListsDataSourceTests.swift */,
+ F01F38212EE1DE2D00A4DC17 /* LocationNodeResolverTests.swift */,
7A9BE5A12B8F88C500E2A7D0 /* LocationNodeTests.swift */,
F0F313A62E85320D00D55C43 /* RecentConnectionsRepositoryTests.swift */,
);
@@ -4826,6 +4844,8 @@
F9C579C52E8FE0D000C90C50 /* LocationDisclosureGroup.swift */,
F9C579C32E8FE08600C90C50 /* LocationListItem.swift */,
F95AC9E92E5DFAFF00A55B52 /* LocationsListView.swift */,
+ F02BFFCC2ED89872007CEA69 /* RecentLocationListItem.swift */,
+ F02BFFCE2ED8997D007CEA69 /* RecentLocationsListView.swift */,
F9C579C72E8FE10400C90C50 /* RelayItemView.swift */,
F90052512E6B06AA0085C80E /* SelectLocationView.swift */,
);
@@ -4866,12 +4886,15 @@
isa = PBXGroup;
children = (
F050AE5F2B73A41E003F4EDB /* AllLocationDataSource.swift */,
+ F02844E12ECF5D9A007EC8F2 /* LocationNodeResolver.swift */,
F04413602BA45CD70018A6EE /* CustomListLocationNodeBuilder.swift */,
F050AE612B74DBAC003F4EDB /* CustomListsDataSource.swift */,
F050AE5D2B739A73003F4EDB /* LocationDataSourceProtocol.swift */,
7A6652B62BB44B120042D848 /* LocationDiffableDataSourceProtocol.swift */,
7A6389F72B864CDF008E77E1 /* LocationNode.swift */,
7A5468AB2C6A55B100590086 /* LocationRelays.swift */,
+ F09444122ECCA4990059606A /* RecentListDataSource.swift */,
+ F02BFFC72ED5BBEB007CEA69 /* RecentsInteractor.swift */,
F01DAE322C2B032A00521E46 /* RelaySelection.swift */,
);
path = DataSource;
@@ -5989,6 +6012,7 @@
A9A5FA0E2ACB05160083449F /* StorePaymentObserver.swift in Sources */,
7A6811542DC8EC6E009CB61A /* UIFont+Weight.swift in Sources */,
A9A5FA102ACB05160083449F /* PacketTunnelAPITransport.swift in Sources */,
+ F02844E32ECF5DB2007EC8F2 /* LocationNodeResolver.swift in Sources */,
7AD63A472CDA666100445268 /* UIntTests.swift in Sources */,
A9A5FA112ACB05160083449F /* APITransportMonitor.swift in Sources */,
A9B6AC1A2ADE8FBB00F7802A /* InMemorySettingsStore.swift in Sources */,
@@ -6036,6 +6060,7 @@
A9A5FA252ACB05160083449F /* UpdateAccountDataOperation.swift in Sources */,
A9A5FA262ACB05160083449F /* UpdateDeviceDataOperation.swift in Sources */,
A9A5FA272ACB05160083449F /* VPNConnectionProtocol.swift in Sources */,
+ F09444142ECCA4B30059606A /* RecentListDataSource.swift in Sources */,
A9A5FA282ACB05160083449F /* WgKeyRotation.swift in Sources */,
449872E42B7CB96300094DDC /* TunnelSettingsUpdateTests.swift in Sources */,
A9A5FA292ACB05160083449F /* AddressCacheTests.swift in Sources */,
@@ -6056,6 +6081,7 @@
7AA513862BC91C6B00D081A4 /* LogRotationTests.swift in Sources */,
F04413622BA45CE30018A6EE /* CustomListLocationNodeBuilder.swift in Sources */,
A9A5FA302ACB05160083449F /* InputTextFormatterTests.swift in Sources */,
+ F01F38222EE1DE3400A4DC17 /* LocationNodeResolverTests.swift in Sources */,
44B02E3B2BC5732D008EDF34 /* LoggingTests.swift in Sources */,
F0B0E6972AFE6E7E001DC66B /* XCTest+Async.swift in Sources */,
F0FA16102D7F2FC7007E2546 /* RelayFilterViewModelTests.swift in Sources */,
@@ -6289,6 +6315,7 @@
F910A4012D3FF23A002FF3BB /* View+Modifier.swift in Sources */,
7A6389EB2B7FAD7A008E77E1 /* SettingsFieldValidationErrorContentView.swift in Sources */,
440870822D7A00B70038972F /* UIImage+Assets.swift in Sources */,
+ F09444132ECCA4B30059606A /* RecentListDataSource.swift in Sources */,
F91CCBF82DFABB75007F1925 /* DeviceManagementView.swift in Sources */,
7A8A19282CF603EB000BCB5B /* SettingsViewControllerFactory.swift in Sources */,
58B26E2A2943545A00D5980C /* NotificationManagerDelegate.swift in Sources */,
@@ -6339,6 +6366,7 @@
58C774BE29A7A249003A1A56 /* CustomNavigationController.swift in Sources */,
E1FD0DF528AA7CE400299DB4 /* StatusActivityView.swift in Sources */,
7A2960FD2A964BB700389B82 /* AlertPresentation.swift in Sources */,
+ F02BFFCF2ED89996007CEA69 /* RecentLocationsListView.swift in Sources */,
0697D6E728F01513007A9E99 /* APITransportMonitor.swift in Sources */,
58968FAE28743E2000B799DC /* TunnelInteractor.swift in Sources */,
7A1A26472A29CF0800B978AA /* RelayFilterDataSource.swift in Sources */,
@@ -6373,6 +6401,7 @@
7A9CCCBD2A96302800DD6A34 /* LoginCoordinator.swift in Sources */,
7A7B3AB62C6DE4DA00D4BCCE /* RestorePurchasesView.swift in Sources */,
7A0EAE9E2D01BCBF00D3EB8B /* View+Size.swift in Sources */,
+ F02BFFC92ED5BBF4007CEA69 /* RecentsInteractor.swift in Sources */,
58293FB125124117005D0BB5 /* CustomTextField.swift in Sources */,
F09A29822A9F8AD200EA3B6F /* RedeemVoucherInteractor.swift in Sources */,
58138E61294871C600684F0C /* DeviceDataThrottling.swift in Sources */,
@@ -6382,6 +6411,7 @@
7A818F1F29F0305800C7F0F4 /* RootConfiguration.swift in Sources */,
F91CCBFA2DFAC8ED007F1925 /* DeviceListView.swift in Sources */,
7A9CCCBF2A96302800DD6A34 /* SettingsCoordinator.swift in Sources */,
+ F02BFFCD2ED89886007CEA69 /* RecentLocationListItem.swift in Sources */,
58F70FE52AEA707800E6890E /* StoreTransactionLog.swift in Sources */,
F9394EF02DC0B58D009595EA /* MullvadListNavigationItemView.swift in Sources */,
582AE3102440A6CA00E6733A /* InputTextFormatter.swift in Sources */,
@@ -6393,6 +6423,7 @@
5823FA5426CE49F700283BF8 /* TunnelObserver.swift in Sources */,
F006CCFC2B99CC8400C6C2AC /* EditLocationsCoordinator.swift in Sources */,
58293FB3251241B4005D0BB5 /* CustomTextView.swift in Sources */,
+ F02BFFCB2ED5D3CB007CEA69 /* MullvadListSectionFooter.swift in Sources */,
586A950E290125F3007BAF2B /* ProductsRequestOperation.swift in Sources */,
7AF9BE902A39F26000DBFEDB /* Collection+Sorting.swift in Sources */,
F0B495762D02025200CFEC2A /* ChipContainerView.swift in Sources */,
@@ -6573,6 +6604,7 @@
F0C6A8432AB08E54000777A8 /* RedeemVoucherViewConfiguration.swift in Sources */,
7AF10EB42ADE85BC00C090B9 /* RelayFilterCoordinator.swift in Sources */,
58FB865526E8BF3100F188BC /* StorePaymentManagerError.swift in Sources */,
+ F02844E22ECF5DB2007EC8F2 /* LocationNodeResolver.swift in Sources */,
F09D04B32AE919AC003D4F89 /* OutgoingConnectionProxy.swift in Sources */,
7A5869BF2B57D0A100640D27 /* IPOverrideStatus.swift in Sources */,
44E1F7582D3EA83A003A60FF /* DestinationDescriber.swift in Sources */,
@@ -9709,7 +9741,7 @@
CURRENT_PROJECT_VERSION = 1;
DEFINES_MODULE = YES;
DEVELOPMENT_TEAM = "";
- "DEVELOPMENT_TEAM[sdk=iphoneos*]" = CKG9MXH72F;
+ "DEVELOPMENT_TEAM[sdk=iphoneos*]" = "";
DYLIB_COMPATIBILITY_VERSION = 1;
DYLIB_CURRENT_VERSION = 1;
DYLIB_INSTALL_NAME_BASE = "@rpath";
@@ -9757,7 +9789,7 @@
CURRENT_PROJECT_VERSION = 1;
DEFINES_MODULE = YES;
DEVELOPMENT_TEAM = "";
- "DEVELOPMENT_TEAM[sdk=iphoneos*]" = CKG9MXH72F;
+ "DEVELOPMENT_TEAM[sdk=iphoneos*]" = "";
DYLIB_COMPATIBILITY_VERSION = 1;
DYLIB_CURRENT_VERSION = 1;
DYLIB_INSTALL_NAME_BASE = "@rpath";
@@ -9853,7 +9885,7 @@
CURRENT_PROJECT_VERSION = 1;
DEFINES_MODULE = YES;
DEVELOPMENT_TEAM = "";
- "DEVELOPMENT_TEAM[sdk=iphoneos*]" = CKG9MXH72F;
+ "DEVELOPMENT_TEAM[sdk=iphoneos*]" = "";
"DEVELOPMENT_TEAM[sdk=macosx*]" = CKG9MXH72F;
DYLIB_COMPATIBILITY_VERSION = 1;
DYLIB_CURRENT_VERSION = 1;
diff --git a/ios/MullvadVPN/AppDelegate.swift b/ios/MullvadVPN/AppDelegate.swift
index 8824a166fe..7537aab95d 100644
--- a/ios/MullvadVPN/AppDelegate.swift
+++ b/ios/MullvadVPN/AppDelegate.swift
@@ -615,11 +615,14 @@ class AppDelegate: UIResponder, UIApplicationDelegate, UNUserNotificationCenterD
let locationService = DefaultLocationService(
urlSession: URLSession.shared, relayCache: cachedRelays)
let locationIdentifier = try? await locationService.fetchCurrentLocationIdentifier()
+ let userSelectedRelays: UserSelectedRelays =
+ if let country = locationIdentifier?.country {
+ UserSelectedRelays(locations: [.country(country)])
+ } else {
+ .default
+ }
- let constraint = RelayConstraint.only(
- UserSelectedRelays(
- locations: [.country(locationIdentifier?.country ?? "se")]
- ))
+ let constraint = RelayConstraint.only(userSelectedRelays)
if !self.appPreferences.hasDoneFirstTimeLogin {
self.tunnelManager.updateSettings([
diff --git a/ios/MullvadVPN/Classes/AccessbilityIdentifier.swift b/ios/MullvadVPN/Classes/AccessbilityIdentifier.swift
index bd909128e3..302066ed2d 100644
--- a/ios/MullvadVPN/Classes/AccessbilityIdentifier.swift
+++ b/ios/MullvadVPN/Classes/AccessbilityIdentifier.swift
@@ -73,6 +73,8 @@ public enum AccessibilityIdentifier: Equatable {
case locationListItem(String)
case entryLocationButton
case exitLocationButton
+ case recentConnectionsToggleButton
+ case disableRecentConnectionsButton
// Cells
case deviceCell
diff --git a/ios/MullvadVPN/Coordinators/ApplicationCoordinator.swift b/ios/MullvadVPN/Coordinators/ApplicationCoordinator.swift
index f135633a3e..f36439d6a4 100644
--- a/ios/MullvadVPN/Coordinators/ApplicationCoordinator.swift
+++ b/ios/MullvadVPN/Coordinators/ApplicationCoordinator.swift
@@ -530,7 +530,8 @@ final class ApplicationCoordinator: Coordinator, Presenting, @preconcurrency Roo
navigationController: navigationController,
tunnelManager: tunnelManager,
relaySelectorWrapper: relaySelectorWrapper,
- customListRepository: CustomListRepository()
+ customListRepository: CustomListRepository(),
+ recentConnectionsRepository: RecentConnectionsRepository(store: SettingsManager.store, maxLimit: 50)
)
locationCoordinator.didFinish = { [weak self] _ in
diff --git a/ios/MullvadVPN/Coordinators/CustomLists/AddLocationsDataSource.swift b/ios/MullvadVPN/Coordinators/CustomLists/AddLocationsDataSource.swift
index 7f22e517b5..8743eafe09 100644
--- a/ios/MullvadVPN/Coordinators/CustomLists/AddLocationsDataSource.swift
+++ b/ios/MullvadVPN/Coordinators/CustomLists/AddLocationsDataSource.swift
@@ -36,7 +36,7 @@ class AddLocationsDataSource:
allLocations: self.nodes
).customListLocationNode
- let sections: [LocationSection] = [.customLists]
+ let sections: [LocationSection] = [.main]
self.sections = sections
super.init(tableView: tableView) { _, indexPath, itemIdentifier in
@@ -60,7 +60,7 @@ class AddLocationsDataSource:
var locationsList: [LocationCellViewModel] = []
nodes.forEach { node in
let viewModel = LocationCellViewModel(
- section: .customLists,
+ section: .main,
node: node,
isSelected: customListLocationNode.children.contains(node)
)
@@ -83,7 +83,7 @@ class AddLocationsDataSource:
locationsList.append(
contentsOf: recursivelyCreateCellViewModelTree(
for: node,
- in: .customLists,
+ in: .main,
indentationLevel: 1
))
}
diff --git a/ios/MullvadVPN/Coordinators/LocationCoordinator.swift b/ios/MullvadVPN/Coordinators/LocationCoordinator.swift
index b19f39e92c..8f69fb68dd 100644
--- a/ios/MullvadVPN/Coordinators/LocationCoordinator.swift
+++ b/ios/MullvadVPN/Coordinators/LocationCoordinator.swift
@@ -17,6 +17,7 @@ class LocationCoordinator: Coordinator, Presentable, Presenting {
private var tunnelObserver: TunnelObserver?
private let relaySelectorWrapper: RelaySelectorWrapper
private let customListRepository: CustomListRepositoryProtocol
+ private let recentConnectionsRepository: RecentConnectionsRepositoryProtocol
let navigationController: UINavigationController
@@ -32,12 +33,14 @@ class LocationCoordinator: Coordinator, Presentable, Presenting {
navigationController: UINavigationController,
tunnelManager: TunnelManager,
relaySelectorWrapper: RelaySelectorWrapper,
- customListRepository: CustomListRepositoryProtocol
+ customListRepository: CustomListRepositoryProtocol,
+ recentConnectionsRepository: RecentConnectionsRepositoryProtocol
) {
self.navigationController = navigationController
self.tunnelManager = tunnelManager
self.relaySelectorWrapper = relaySelectorWrapper
self.customListRepository = customListRepository
+ self.recentConnectionsRepository = recentConnectionsRepository
}
func start() {
@@ -45,6 +48,7 @@ class LocationCoordinator: Coordinator, Presentable, Presenting {
tunnelManager: tunnelManager,
relaySelectorWrapper: relaySelectorWrapper,
customListRepository: customListRepository,
+ recentConnectionsRepository: recentConnectionsRepository,
delegate: .init(
showDaitaSettings: { [weak self] in
self?.navigateToDaitaSettings()
@@ -71,7 +75,7 @@ class LocationCoordinator: Coordinator, Presentable, Presenting {
didSelectExitRelayLocations: { [weak self] relays in
guard let self else { return }
self.didSelectExitRelays(relays)
- self.didFinish?(self)
+ //self.didFinish?(self)
},
didSelectEntryRelayLocations: { [weak self] relays in
self?.didSelectEntryRelays(relays)
diff --git a/ios/MullvadVPN/View controllers/AccountDeletion/AccountDeletionView.swift b/ios/MullvadVPN/View controllers/AccountDeletion/AccountDeletionView.swift
index 5792e0a482..97125a2e16 100644
--- a/ios/MullvadVPN/View controllers/AccountDeletion/AccountDeletionView.swift
+++ b/ios/MullvadVPN/View controllers/AccountDeletion/AccountDeletionView.swift
@@ -41,7 +41,7 @@ struct AccountDeletionView: View {
// accountTextField
let placeholder = "XXXX"
MullvadPrimaryTextField(
- label: "Last 4 digits",
+ label: LocalizedStringKey("Last 4 digits"),
placeholder: LocalizedStringKey(placeholder),
text: $viewModel.enteredAccountNumberSuffix,
keyboardType: .numberPad
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/AllLocationDataSource.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/AllLocationDataSource.swift
index 333e028027..becd512def 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/AllLocationDataSource.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/AllLocationDataSource.swift
@@ -111,4 +111,12 @@ class AllLocationDataSource: LocationDataSourceProtocol {
}
}
}
+
+ func node(by selectedRelays: UserSelectedRelays) -> LocationNode? {
+ let rootNode = RootLocationNode(children: nodes)
+ guard let location = selectedRelays.locations.first else {
+ return nil
+ }
+ return descendantNode(in: rootNode, for: location, baseCodes: [])
+ }
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/CustomListsDataSource.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/CustomListsDataSource.swift
index 59c8ae21a5..4ce1c2edf9 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/CustomListsDataSource.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/CustomListsDataSource.swift
@@ -40,4 +40,29 @@ class CustomListsDataSource: LocationDataSourceProtocol {
return listNode
}
}
+
+ func node(by selectedRelays: UserSelectedRelays) -> LocationNode? {
+ let rootNode = RootLocationNode(children: nodes)
+ guard
+ let selection = selectedRelays.customListSelection,
+ let selectedNode = rootNode.children.first(where: {
+ $0.asCustomListNode?.customList.id == selection.listId
+ })
+ else { return nil }
+
+ if selection.isList {
+ return selectedNode
+ }
+
+ if let location = selectedRelays.locations.first {
+ return descendantNode(
+ in: rootNode,
+ for: location,
+ baseCodes: [selectedNode.code]
+ )
+ }
+
+ return nil
+ }
+
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationDataSourceProtocol.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationDataSourceProtocol.swift
index 057a82114c..dc99cef7e0 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationDataSourceProtocol.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationDataSourceProtocol.swift
@@ -12,61 +12,10 @@ import MullvadTypes
protocol LocationDataSourceProtocol {
var nodes: [LocationNode] { get }
+ func node(by selectedRelays: UserSelectedRelays) -> LocationNode?
}
extension LocationDataSourceProtocol {
-
- func setConnectedRelay(hostname: String?) {
- nodes.forEachNode { node in
- node.isConnected = node.name == hostname
- }
- }
-
- /// Excludeds nodes from being selectable. A node gets excluded if the selection only allows for one possible relay.
- /// This is used in multihop to make sure that the during relay selection entry and exit can different.
- /// It prevent the user from making a selection that would lead to the blocked state.
- /// - Parameters:
- /// - excludedSelection: The selection that should be checked for exclusion.
- func setExcludedNode(excludedSelection: UserSelectedRelays?) {
- nodes.forEachNode { node in
- node.isExcluded = false
- }
- guard let selectedRelayLocations = excludedSelection?.locations,
- selectedRelayLocations.count == 1,
- let selectedRelayLocation = selectedRelayLocations.first
- else {
- return
- }
- nodes.forEachNode { node in
- let locations = Set((node.flattened + [node]).flatMap { $0.locations })
- if locations
- .contains(selectedRelayLocation) && node.activeRelayNodes.count == 1
- {
- node.isExcluded = true
- node.forEachDescendant { child in
- child.isExcluded = true
- }
- }
- }
- }
-
- func setSelectedNode(selectedRelays: UserSelectedRelays?) {
- nodes.forEachNode { node in
- node.isSelected = false
- }
- guard let selectedRelays else { return }
- let selectedNode = node(by: selectedRelays)
- selectedNode?.isSelected = true
- }
-
- func expandSelection() {
- nodes.forEachNode { node in
- if node.isSelected {
- node.forEachAncestor { $0.showsChildren = true }
- }
- }
- }
-
func search(by text: String) {
nodes.forEachNode { node in
node.isHiddenFromSearch = false
@@ -100,37 +49,22 @@ extension LocationDataSourceProtocol {
return node.isHiddenFromSearch
}
- func node(by selectedRelays: UserSelectedRelays) -> LocationNode? {
- let rootNode = RootLocationNode(children: nodes)
+ func descendantNode(
+ in rootNode: LocationNode,
+ for location: RelayLocation,
+ baseCodes: [String]
+ ) -> LocationNode? {
+ let extraCodes: [String]
+ switch location {
+ case let .country(countryCode):
+ extraCodes = [countryCode]
+ case let .city(countryCode, cityCode):
+ extraCodes = [countryCode, cityCode]
- let descendantNodeFor: ([String]) -> LocationNode? = { codes in
- guard let location = selectedRelays.locations.first else {
- return nil
- }
- return switch location {
- case let .country(countryCode):
- rootNode.descendantNodeFor(codes: codes + [countryCode])
- case let .city(countryCode, cityCode):
- rootNode.descendantNodeFor(codes: codes + [countryCode, cityCode])
- case let .hostname(_, _, hostCode):
- rootNode.descendantNodeFor(codes: codes + [hostCode])
- }
+ case let .hostname(_, _, hostCode):
+ extraCodes = [hostCode]
}
- if let customListSelection = selectedRelays.customListSelection {
- let selectedCustomListNode = nodes.first(where: {
- $0.asCustomListNode?.customList.id == customListSelection.listId
- })
-
- guard let selectedCustomListNode else { return nil }
-
- if customListSelection.isList {
- return selectedCustomListNode
- }
-
- return descendantNodeFor([selectedCustomListNode.code])
- } else {
- return descendantNodeFor([])
- }
+ return rootNode.descendantNodeFor(codes: baseCodes + extraCodes)
}
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNode.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNode.swift
index 59db93fa9c..d29376cf94 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNode.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNode.swift
@@ -47,6 +47,28 @@ class LocationNode: @unchecked Sendable {
self.isSelected = isSelected
self.isExcluded = isExcluded
}
+
+ /// Recursively copies a node, its parent and its descendants from another
+ /// node (tree), with an optional custom root parent.
+ func copy(withParent parent: LocationNode? = nil) -> LocationNode {
+ let node = LocationNode(
+ name: name,
+ code: code,
+ locations: locations,
+ isActive: isActive,
+ parent: parent,
+ children: [],
+ showsChildren: showsChildren,
+ isHiddenFromSearch: isHiddenFromSearch,
+ isConnected: isConnected,
+ isSelected: false, // explicity set to false since it's a different node
+ isExcluded: isExcluded
+ )
+
+ node.children = recursivelyCopyChildren(withParent: node)
+
+ return node
+ }
}
extension LocationNode {
@@ -125,36 +147,11 @@ extension LocationNode {
}
}
}
-}
-
-extension LocationNode {
- /// Recursively copies a node, its parent and its descendants from another
- /// node (tree), with an optional custom root parent.
- func copy(withParent parent: LocationNode? = nil) -> LocationNode {
- let node = LocationNode(
- name: name,
- code: code,
- locations: locations,
- isActive: isActive,
- parent: parent,
- children: [],
- showsChildren: showsChildren,
- isHiddenFromSearch: isHiddenFromSearch,
- isConnected: isConnected,
- isSelected: false, // explicity set to false since it's a different node
- isExcluded: isExcluded
- )
-
- node.children = recursivelyCopyChildren(withParent: node)
-
- return node
- }
- private func recursivelyCopyChildren(withParent parent: LocationNode) -> [LocationNode] {
+ fileprivate func recursivelyCopyChildren(withParent parent: LocationNode) -> [LocationNode] {
children.map { $0.copy(withParent: parent) }
}
}
-
extension LocationNode: Hashable {
func hash(into hasher: inout Hasher) {
hasher.combine(code)
@@ -214,4 +211,23 @@ class CustomListLocationNode: LocationNode, @unchecked Sendable {
isHiddenFromSearch: isHiddenFromSearch
)
}
+
+ override func copy(withParent parent: LocationNode? = nil) -> LocationNode {
+ let node = CustomListLocationNode(
+ name: name,
+ code: code,
+ locations: locations,
+ isActive: isActive,
+ parent: parent,
+ children: [],
+ showsChildren: showsChildren,
+ isHiddenFromSearch: isHiddenFromSearch,
+ customList: customList
+ )
+
+ // Copy children recursively
+ node.children = recursivelyCopyChildren(withParent: node)
+
+ return node
+ }
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNodeResolver.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNodeResolver.swift
new file mode 100644
index 0000000000..f8a0509d01
--- /dev/null
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/LocationNodeResolver.swift
@@ -0,0 +1,73 @@
+//
+// LocationNodeResolver.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-11-20.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+
+import MullvadTypes
+
+final class LocationNodeResolver {
+ private let providers: [LocationDataSourceProtocol]
+ private var selectedNode: LocationNode?
+
+ init(providers: [LocationDataSourceProtocol]) {
+ self.providers = providers
+ }
+
+ func setSelectedNodeExpanded(_ isExpanded: Bool) {
+ selectedNode?.forEachAncestor { $0.showsChildren = isExpanded }
+ }
+
+ func setSelectedNode(selectedRelays: UserSelectedRelays) {
+ resetSelection()
+ selectedNode = node(by: selectedRelays)
+ selectedNode?.isSelected = true
+ }
+
+ func setConnectedRelay(hostname: String?) {
+ selectedNode?.flattened.forEachNode { node in
+ node.isConnected = node.name == hostname
+ }
+ }
+
+ /// Excluded nodes from being selectable. A node gets excluded if the selection only allows for one possible relay.
+ /// This is used in multihop to make sure that the during relay selection entry and exit can different.
+ /// It prevent the user from making a selection that would lead to the blocked state.
+ /// - Parameters:
+ /// - excludedSelection: The selection that should be checked for exclusion.
+ func setExcludedNode(excludedSelection: UserSelectedRelays?) {
+ guard let excludedSelection,
+ let excludedNode = node(by: excludedSelection)
+ else { return }
+ resetExclusion()
+ excludedNode.isExcluded = excludedNode.activeRelayNodes.count == 1
+ }
+
+ private func node(by relays: UserSelectedRelays) -> LocationNode? {
+ for provider in providers {
+ if let node = provider.node(by: relays) {
+ return node
+ }
+ }
+ return nil
+ }
+
+ private func resetSelection() {
+ providers.forEach { provider in
+ provider.nodes.forEachNode { node in
+ node.isSelected = false
+ node.showsChildren = false
+ }
+ }
+ }
+
+ private func resetExclusion() {
+ providers.forEach { provider in
+ provider.nodes.forEachNode { node in
+ node.isExcluded = false
+ }
+ }
+ }
+}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentListDataSource.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentListDataSource.swift
new file mode 100644
index 0000000000..8e7fc09d89
--- /dev/null
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentListDataSource.swift
@@ -0,0 +1,48 @@
+//
+// RecentListDataSource.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-11-18.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+
+import Foundation
+import MullvadLogging
+import MullvadSettings
+import MullvadTypes
+
+class RecentListDataSource: LocationDataSourceProtocol {
+ private(set) var nodes = [LocationNode]()
+ let allLocationDataSource: AllLocationDataSource
+ let customListsDataSource: CustomListsDataSource
+
+ init(_ allLocationDataSource: AllLocationDataSource, customListsDataSource: CustomListsDataSource) {
+ self.allLocationDataSource = allLocationDataSource
+ self.customListsDataSource = customListsDataSource
+ }
+
+ func reload(_ recents: [UserSelectedRelays]) {
+ // Resolve each recent selection to a location node by checking custom lists first,
+ // then falling back to the main location data source. Convert each resolved node
+ // and keep only the first three results.
+ self.nodes = Array(
+ recents.map({ (userSelectedRelays) -> LocationNode? in
+ customListsDataSource.node(by: userSelectedRelays)
+ ?? allLocationDataSource.node(by: userSelectedRelays)
+ })
+ .compactMap({ $0?.copy() })
+ .prefix(3))
+ }
+
+ func node(by selectedRelays: UserSelectedRelays) -> LocationNode? {
+ self.nodes.first(where: {
+ let userSelectedRelays = $0.userSelectedRelays
+ if let customListSelection = userSelectedRelays.customListSelection,
+ customListSelection.isList
+ {
+ return userSelectedRelays.locations == $0.locations
+ }
+ return userSelectedRelays.locations == $0.locations
+ })
+ }
+}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentsInteractor.swift b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentsInteractor.swift
new file mode 100644
index 0000000000..82547c2aa7
--- /dev/null
+++ b/ios/MullvadVPN/View controllers/SelectLocation/DataSource/RecentsInteractor.swift
@@ -0,0 +1,110 @@
+//
+// RecentsInteractor.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-11-25.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+
+import Combine
+import MullvadLogging
+import MullvadSettings
+import MullvadTypes
+
+protocol RecentsInteractorProtocol {
+ var isEnabledPublisher: AnyPublisher<Bool, Never> { get }
+ func toggle()
+ func fetch(context: MultihopContext) -> [UserSelectedRelays]
+ func updateSelectedLocations(_ selectedLocations: UserSelectedRelays, for context: MultihopContext)
+}
+
+class RecentsInteractor: RecentsInteractorProtocol {
+ private let repository: RecentConnectionsRepositoryProtocol
+ private var selectedEntryRelays: UserSelectedRelays?
+ private var selectedExitRelays: UserSelectedRelays
+ private let logger = Logger(label: "RecentsInteractor")
+ private var recentConnections: RecentConnections?
+ private var cancellables = Set<Combine.AnyCancellable>()
+ private var isEnabledSubject = CurrentValueSubject<Bool, Never>(false)
+ var isEnabledPublisher: AnyPublisher<Bool, Never> {
+ isEnabledSubject.eraseToAnyPublisher()
+ }
+
+ init(
+ selectedEntryRelays: UserSelectedRelays?,
+ selectedExitRelays: UserSelectedRelays,
+ repository: RecentConnectionsRepositoryProtocol
+ ) {
+ self.repository = repository
+ self.selectedEntryRelays = selectedEntryRelays
+ self.selectedExitRelays = selectedExitRelays
+ self.subscribeToRecentConnections()
+ self.repository.initiate()
+ }
+
+ private func subscribeToRecentConnections() {
+ repository
+ .recentConnectionsPublisher
+ .sink(receiveValue: { [weak self] result in
+ guard let self else { return }
+ switch result {
+ case .success(let value):
+ recentConnections = value
+ isEnabledSubject.send(value.isEnabled)
+ case .failure(let error) where (error as? KeychainError) == .itemNotFound:
+ // Key not found: this occurs only on first use.
+ // Initialize Recents using the user's most recent entry/exit selections by default.
+ repository.enable(selectedEntryRelays, selectedExitRelays: selectedExitRelays)
+ case .failure(let error):
+ logger.error("Failed to subscribe to recent connections: \(error)")
+ }
+ })
+ .store(in: &cancellables)
+ }
+
+ func updateSelectedLocations(_ selectedLocations: UserSelectedRelays, for context: MultihopContext) {
+ updateRelays(selectedLocations, for: context)
+ guard isEnabled else { return }
+ persistRelaySelection()
+ }
+
+ var isEnabled: Bool {
+ recentConnections?.isEnabled ?? false
+ }
+
+ func fetch(context: MultihopContext) -> [UserSelectedRelays] {
+ switch context {
+ case .entry:
+ recentConnections?.entryLocations ?? []
+ case .exit:
+ recentConnections?.exitLocations ?? []
+ }
+ }
+
+ func toggle() {
+ if isEnabled {
+ repository.disable()
+ } else {
+ repository.enable(selectedEntryRelays, selectedExitRelays: selectedExitRelays)
+ }
+ }
+
+ private func updateRelays(
+ _ relays: UserSelectedRelays,
+ for context: MultihopContext
+ ) {
+ switch context {
+ case .entry:
+ selectedEntryRelays = relays
+ case .exit:
+ selectedExitRelays = relays
+ }
+ }
+
+ private func persistRelaySelection() {
+ repository.add(
+ selectedEntryRelays,
+ selectedExitRelays: selectedExitRelays
+ )
+ }
+}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/LocationContext.swift b/ios/MullvadVPN/View controllers/SelectLocation/LocationContext.swift
index 5047ff6362..2fc7c934f6 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/LocationContext.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/LocationContext.swift
@@ -1,10 +1,12 @@
struct LocationContext {
+ var recents: [LocationNode]
var locations: [LocationNode]
var customLists: [LocationNode]
var filter: [SelectLocationFilter]
let selectLocation: (LocationNode) -> Void
init(
+ recents: [LocationNode] = [],
locations: [LocationNode] = [],
customLists: [LocationNode] = [],
filter: [SelectLocationFilter] = [],
@@ -12,6 +14,7 @@ struct LocationContext {
connectedRelayHostname: String? = nil,
selectLocation: @escaping (LocationNode) -> Void = { _ in }
) {
+ self.recents = recents
self.locations = locations
self.customLists = customLists
self.filter = filter
@@ -19,7 +22,7 @@ struct LocationContext {
}
var selectedLocation: LocationNode? {
- (locations + customLists)
+ (locations + customLists + recents)
.flatMap { [$0] + $0.flattened }
.first { $0.isSelected }
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/LocationSection.swift b/ios/MullvadVPN/View controllers/SelectLocation/LocationSection.swift
index 8eb2646855..18bc68563e 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/LocationSection.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/LocationSection.swift
@@ -9,26 +9,7 @@
import Foundation
enum LocationSection: String, Hashable, CaseIterable, CellIdentifierProtocol, Sendable {
- case customLists
- case allLocations
-
- var header: String {
- switch self {
- case .customLists:
- return NSLocalizedString("Custom lists", comment: "")
- case .allLocations:
- return NSLocalizedString("All locations", comment: "")
- }
- }
-
- var footer: String {
- switch self {
- case .customLists:
- return NSLocalizedString("To create a custom list, tap on \"...\" ", comment: "")
- case .allLocations:
- return NSLocalizedString("No matching relays found, check your filter settings.", comment: "")
- }
- }
+ case main
var cellClass: AnyClass {
LocationCell.self
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/MockSelectLocationViewModel.swift b/ios/MullvadVPN/View controllers/SelectLocation/MockSelectLocationViewModel.swift
index a55f86895d..de32a5b84c 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/MockSelectLocationViewModel.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/MockSelectLocationViewModel.swift
@@ -1,6 +1,9 @@
import Foundation
class MockSelectLocationViewModel: SelectLocationViewModel {
+
+ var isRecentsEnabled: Bool = true
+
var isMultihopEnabled: Bool = true
var showDAITAInfo = false
@@ -13,6 +16,17 @@ class MockSelectLocationViewModel: SelectLocationViewModel {
init() {
entryContext = LocationContext()
exitContext = LocationContext(
+ recents: [
+ LocationNode(
+ name: "Stockholm",
+ code: "sth",
+ children: [
+ LocationNode(name: sth1, code: sth1),
+ LocationNode(name: sth2, code: sth2),
+ LocationNode(name: sth3, code: sth3),
+ ]
+ )
+ ],
locations: [
LocationNode(
name: "Sweden", code: "se",
@@ -176,6 +190,8 @@ class MockSelectLocationViewModel: SelectLocationViewModel {
func expandSelectedLocation() {}
+ func toggleRecents() {}
+
private var got1 = "se-got-001"
private let got2 = "se-got-002"
private let got3 = "se-got-003"
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/SelectLocationViewModel.swift b/ios/MullvadVPN/View controllers/SelectLocation/SelectLocationViewModel.swift
index cc78eb3151..fbea69110e 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/SelectLocationViewModel.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/SelectLocationViewModel.swift
@@ -11,6 +11,7 @@ protocol SelectLocationViewModel: ObservableObject {
var searchText: String { get set }
var showDAITAInfo: Bool { get }
var isMultihopEnabled: Bool { get }
+ var isRecentsEnabled: Bool { get }
func onFilterTapped(_ filter: SelectLocationFilter)
func onFilterRemoved(_ filter: SelectLocationFilter)
func customListsChanged()
@@ -23,6 +24,7 @@ protocol SelectLocationViewModel: ObservableObject {
func showEditCustomListView(locations: [LocationNode])
func showAddCustomListView(locations: [LocationNode])
func showFilterView()
+ func toggleRecents()
}
struct SelectLocationDelegate {
@@ -39,8 +41,8 @@ struct SelectLocationDelegate {
@MainActor
class SelectLocationViewModelImpl: SelectLocationViewModel {
@Published var isMultihopEnabled: Bool
+ @Published var isRecentsEnabled: Bool = true
@Published var multihopContext: MultihopContext = .exit
-
@Published var exitContext = LocationContext()
@Published var entryContext = LocationContext()
@Published var searchText: String = ""
@@ -50,17 +52,23 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
private let entryLocationsDataSource = AllLocationDataSource()
private let entryCustomListsDataSource: CustomListsDataSource
private let exitCustomListsDataSource: CustomListsDataSource
+ private let entryRecentsDataSource: RecentListDataSource
+ private let exitRecentsDataSource: RecentListDataSource
private let relaySelectorWrapper: RelaySelectorWrapper
private let tunnelManager: TunnelManager
private let customListInteractor: CustomListInteractorProtocol
+ private let recentsInteractor: RecentsInteractorProtocol
private var relaysCandidates: RelayCandidates?
private var tunnelObserver: TunnelBlockObserver?
private let delegate: SelectLocationDelegate
- private var cancellables: [Combine.AnyCancellable] = []
+ private var cancellables = Set<Combine.AnyCancellable>()
+
+ let entryLocationsProvider: LocationNodeResolver
+ let exitLocationsProvider: LocationNodeResolver
private var allLocations: [LocationNode] {
exitContext.locations + exitContext.customLists + entryContext.locations + entryContext.customLists
@@ -70,6 +78,7 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
tunnelManager: TunnelManager,
relaySelectorWrapper: RelaySelectorWrapper,
customListRepository: CustomListRepositoryProtocol,
+ recentConnectionsRepository: RecentConnectionsRepositoryProtocol,
delegate: SelectLocationDelegate
) {
self.tunnelManager = tunnelManager
@@ -78,6 +87,11 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
tunnelManager: tunnelManager,
repository: customListRepository
)
+ self.recentsInteractor = RecentsInteractor(
+ selectedEntryRelays: tunnelManager.settings.selectedEntryRelays,
+ selectedExitRelays: tunnelManager.settings.selectedExitRelays,
+ repository: recentConnectionsRepository)
+
self.delegate = delegate
self.entryCustomListsDataSource = CustomListsDataSource(
repository: customListRepository
@@ -86,11 +100,44 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
repository: customListRepository
)
+ self.entryRecentsDataSource = RecentListDataSource(
+ entryLocationsDataSource, customListsDataSource: entryCustomListsDataSource)
+ self.exitRecentsDataSource = RecentListDataSource(
+ exitLocationsDataSource, customListsDataSource: exitCustomListsDataSource)
+
+ entryLocationsProvider = LocationNodeResolver(
+ providers: [
+ entryRecentsDataSource,
+ entryCustomListsDataSource,
+ entryLocationsDataSource,
+ ]
+ )
+
+ exitLocationsProvider = LocationNodeResolver(
+ providers: [
+ exitRecentsDataSource,
+ exitCustomListsDataSource,
+ exitLocationsDataSource,
+ ]
+ )
+
showDAITAInfo = tunnelManager.settings.daita.isAutomaticRouting
// If multihop is enabled, we should check if there's a DAITA related error when opening the location
// view. If there is, help the user by showing the entry instead of the exit view.
isMultihopEnabled = tunnelManager.settings.tunnelMultihopState.isEnabled
+
+ // Reactively keep `isRecentsEnabled` in sync with the interactor's enabled state.
+ recentsInteractor
+ .isEnabledPublisher
+ .sink(receiveValue: { [weak self] isEnabled in
+ guard let self else { return }
+ isRecentsEnabled = isEnabled
+ refreshRecents()
+ updateSelections()
+ })
+ .store(in: &cancellables)
+
if isMultihopEnabled {
self.multihopContext =
if case .noRelaysSatisfyingDaitaConstraints = tunnelManager.tunnelStatus.observedState
@@ -101,16 +148,18 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
self.entryContext = LocationContext(
filter: SelectLocationFilter.getActiveFilters(tunnelManager.settings).0,
selectLocation: { [weak self] location in
- delegate
- .didSelectEntryRelayLocations(location.userSelectedRelays)
- self?.multihopContext = .exit
+ guard let self else { return }
+ recentsInteractor.updateSelectedLocations(location.userSelectedRelays, for: .entry)
+ delegate.didSelectEntryRelayLocations(location.userSelectedRelays)
+ multihopContext = .exit
}
)
self.exitContext = LocationContext(
filter: SelectLocationFilter.getActiveFilters(tunnelManager.settings).1,
- selectLocation: { location in
- delegate
- .didSelectExitRelayLocations(location.userSelectedRelays)
+ selectLocation: { [weak self] location in
+ guard let self else { return }
+ recentsInteractor.updateSelectedLocations(location.userSelectedRelays, for: .exit)
+ delegate.didSelectExitRelayLocations(location.userSelectedRelays)
}
)
let tunnelObserver =
@@ -122,10 +171,8 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
guard let self else { return }
fetchLocations()
refreshCustomLists()
- updateSelections(
- selectedExitRelays: settings.relayConstraints.exitLocations.value,
- selectedEntryRelays: settings.relayConstraints.entryLocations.value
- )
+ refreshRecents()
+ updateSelections()
updateConnectedLocations(tunnelManager.tunnelStatus)
if !searchText.isEmpty {
search(searchText: searchText)
@@ -150,7 +197,7 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
if prevValue == nil && newValue == "" { return }
self?.search(searchText: newValue)
if newValue == "" {
- self?.expandSelectedLocation()
+ self?.updateSelections()
}
}.store(in: &cancellables)
@@ -159,12 +206,8 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
fetchLocations()
refreshCustomLists()
- updateSelections(
- selectedExitRelays: tunnelManager.settings.relayConstraints.exitLocations.value,
- selectedEntryRelays: tunnelManager.settings.relayConstraints.entryLocations.value
- )
- updateConnectedLocations(tunnelManager.tunnelStatus)
- expandSelectedLocation()
+ refreshRecents()
+ updateSelections()
}
deinit {
@@ -247,13 +290,18 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
func customListsChanged() {
refreshCustomLists()
- updateSelections(
- selectedExitRelays: tunnelManager.settings.relayConstraints.exitLocations.value,
- selectedEntryRelays: tunnelManager.settings.relayConstraints.entryLocations.value
- )
+ refreshRecents()
+ updateSelections()
updateConnectedLocations(tunnelManager.tunnelStatus)
}
+ private func refreshRecents() {
+ entryRecentsDataSource.reload(recentsInteractor.fetch(context: .entry))
+ exitRecentsDataSource.reload(recentsInteractor.fetch(context: .exit))
+ entryContext.recents = entryRecentsDataSource.nodes
+ exitContext.recents = exitRecentsDataSource.nodes
+ }
+
private func refreshCustomLists() {
exitCustomListsDataSource.reload(allLocationNodes: exitContext.locations)
entryCustomListsDataSource.reload(allLocationNodes: entryContext.locations)
@@ -284,67 +332,39 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
}
private func updateConnectedLocations(_ status: TunnelStatus) {
- exitLocationsDataSource
- .setConnectedRelay(hostname: status.state.relays?.exit.hostname)
- exitCustomListsDataSource
- .setConnectedRelay(hostname: status.state.relays?.exit.hostname)
- entryLocationsDataSource
- .setConnectedRelay(hostname: status.state.relays?.entry?.hostname)
- entryCustomListsDataSource
- .setConnectedRelay(hostname: status.state.relays?.entry?.hostname)
+ entryLocationsProvider.setConnectedRelay(hostname: status.state.relays?.entry?.hostname)
+ exitLocationsProvider.setConnectedRelay(hostname: status.state.relays?.exit.hostname)
}
private func search(searchText: String) {
- exitLocationsDataSource
- .search(by: searchText)
- exitCustomListsDataSource
- .search(by: searchText)
- entryLocationsDataSource
- .search(by: searchText)
- entryCustomListsDataSource
- .search(by: searchText)
+
+ exitLocationsDataSource.search(by: searchText)
+ exitCustomListsDataSource.search(by: searchText)
+ exitRecentsDataSource.search(by: searchText)
+
+ entryLocationsDataSource.search(by: searchText)
+ entryCustomListsDataSource.search(by: searchText)
+ entryRecentsDataSource.search(by: searchText)
}
- private func updateSelections(
- selectedExitRelays: UserSelectedRelays?,
- selectedEntryRelays: UserSelectedRelays?
- ) {
- // set exit selection
- exitLocationsDataSource
- .setSelectedNode(selectedRelays: selectedExitRelays)
- exitCustomListsDataSource
- .setSelectedNode(selectedRelays: selectedExitRelays)
+ private func updateSelections() {
+ let selectedEntryRelays = tunnelManager.settings.selectedEntryRelays
+ let selectedExitRelays = tunnelManager.settings.selectedExitRelays
- if isMultihopEnabled {
- // set entry selection
- entryLocationsDataSource
- .setSelectedNode(selectedRelays: selectedEntryRelays)
- entryCustomListsDataSource
- .setSelectedNode(selectedRelays: selectedEntryRelays)
+ entryLocationsProvider.setSelectedNode(selectedRelays: selectedEntryRelays)
+ exitLocationsProvider.setSelectedNode(selectedRelays: selectedExitRelays)
- // exclude selected entry relays in exit lists
- exitLocationsDataSource
- .setExcludedNode(excludedSelection: selectedEntryRelays)
- exitCustomListsDataSource
- .setExcludedNode(excludedSelection: selectedEntryRelays)
+ entryLocationsProvider.setExcludedNode(excludedSelection: selectedExitRelays)
+ exitLocationsProvider.setExcludedNode(excludedSelection: selectedEntryRelays)
- // exclude selected exit relays in entry lists
- entryLocationsDataSource
- .setExcludedNode(excludedSelection: selectedExitRelays)
- entryCustomListsDataSource
- .setExcludedNode(excludedSelection: selectedExitRelays)
- }
+ updateConnectedLocations(tunnelManager.tunnelStatus)
+
+ setSelectedNodeExpanded(!isRecentsEnabled)
}
- private func expandSelectedLocation() {
- exitLocationsDataSource
- .expandSelection()
- exitCustomListsDataSource
- .expandSelection()
- entryLocationsDataSource
- .expandSelection()
- entryCustomListsDataSource
- .expandSelection()
+ private func setSelectedNodeExpanded(_ isExpanded: Bool) {
+ exitLocationsProvider.setSelectedNodeExpanded(isExpanded)
+ entryLocationsProvider.setSelectedNodeExpanded(isExpanded)
}
func didFinish() {
@@ -366,4 +386,17 @@ class SelectLocationViewModelImpl: SelectLocationViewModel {
func showFilterView() {
delegate.showFilterView()
}
+
+ func toggleRecents() {
+ recentsInteractor.toggle()
+ }
+}
+
+private extension LatestTunnelSettings {
+ var selectedEntryRelays: UserSelectedRelays {
+ relayConstraints.entryLocations.value ?? .default
+ }
+ var selectedExitRelays: UserSelectedRelays {
+ relayConstraints.exitLocations.value ?? .default
+ }
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/EntryLocationView.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/EntryLocationView.swift
index 7fe562d834..50f2ad203e 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/Views/EntryLocationView.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/EntryLocationView.swift
@@ -2,7 +2,7 @@ import SwiftUI
struct EntryLocationView<ViewModel: SelectLocationViewModel>: View {
@ObservedObject var viewModel: ViewModel
- let onShowsTopOfTheListChange: (Bool) -> Void
+ let onScrollOffsetChange: (CGFloat, CGFloat) -> Void
var body: some View {
if viewModel.showDAITAInfo {
DaitaWarningView {
@@ -11,7 +11,7 @@ struct EntryLocationView<ViewModel: SelectLocationViewModel>: View {
} else {
ExitLocationView(
viewModel: viewModel, context: $viewModel.entryContext,
- onShowsTopOfTheListChange: onShowsTopOfTheListChange)
+ onScrollOffsetChange: onScrollOffsetChange)
}
}
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/ExitLocationView.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/ExitLocationView.swift
index 37b6539582..029c9f3f61 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/Views/ExitLocationView.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/ExitLocationView.swift
@@ -5,9 +5,8 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
@Binding var context: LocationContext
@State var newCustomListAlert: MullvadInputAlert?
@State var alert: MullvadAlert?
- let onShowsTopOfTheListChange: (Bool) -> Void
-
- @State private var isShowingTopOfTheList: Bool = false
+ let onScrollOffsetChange: (CGFloat, CGFloat) -> Void
+ @State private var previousScrollOffset: CGFloat = 0
var isShowingCustomListsSection: Bool {
viewModel.searchText.isEmpty
|| (!viewModel.searchText.isEmpty
@@ -20,6 +19,10 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
!context.locations.filter({ !$0.isHiddenFromSearch }).isEmpty
}
+ var isShowingRecentsSection: Bool {
+ !context.recents.filter({ !$0.isHiddenFromSearch }).isEmpty
+ }
+
var body: some View {
ScrollViewReader { scrollProxy in
// All items in the list are arranged in a flat hierarchy
@@ -37,6 +40,9 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
.padding(.bottom, 16)
}
Group {
+ if viewModel.isRecentsEnabled {
+ recentsSection(isShowingRecentsSection: isShowingRecentsSection)
+ }
if isShowingCustomListsSection {
customListSection(isShowingHeader: isShowingAllLocationsSection)
}
@@ -55,16 +61,14 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
.zIndex(3) // prevent wrong overlapping during animations
}
.capturePosition(in: .exitLocationScroll) { frame in
- isShowingTopOfTheList = frame.minY >= 0
- }
- .onChange(of: isShowingTopOfTheList) {
- onShowsTopOfTheListChange(isShowingTopOfTheList)
+ onScrollOffsetChange(previousScrollOffset, frame.minY)
+ previousScrollOffset = frame.minY
}
}
.coordinateSpace(.exitLocationScroll)
.task {
guard viewModel.searchText.isEmpty else { return }
- let selectedLocation = (context.locations + context.customLists)
+ let selectedLocation = (context.locations + context.customLists + context.recents)
.flatMap { $0.flattened + [$0] }
.first { $0.isSelected }
@@ -94,6 +98,25 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
}
@ViewBuilder
+ func recentsSection(isShowingRecentsSection: Bool) -> some View {
+ MullvadListSectionHeader(title: "Recents")
+ if isShowingRecentsSection {
+ RecentLocationsListView(
+ locations: $context.recents,
+ multihopContext: viewModel.multihopContext,
+ ) { location in
+ context.selectLocation(location)
+ } contextMenu: { location in
+ locationContextMenu(location)
+ }
+ } else {
+ MullvadListSectionFooter(title: "No recent selection history")
+ .padding(.horizontal, context.recents.isEmpty ? 0 : 16)
+ .padding(.top, context.recents.isEmpty ? 0 : 4)
+ }
+ }
+
+ @ViewBuilder
func customListSection(isShowingHeader: Bool) -> some View {
if isShowingHeader {
HStack(spacing: 0) {
@@ -137,12 +160,9 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
: """
To add locations to a list, press the pen or long press on a country, city, or server.
"""
- Text(text)
- .font(.mullvadMini)
- .foregroundStyle(Color.mullvadTextPrimary.opacity(0.6))
+ MullvadListSectionFooter(title: text)
.padding(.horizontal, context.customLists.isEmpty ? 0 : 16)
.padding(.top, context.customLists.isEmpty ? 0 : 4)
- .padding(.bottom, 24)
}
}
@@ -153,7 +173,7 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
context: $viewModel.exitContext,
newCustomListAlert: nil,
alert: nil,
- onShowsTopOfTheListChange: { _ in }
+ onScrollOffsetChange: { _, _ in }
)
.background(Color.mullvadBackground)
}
@@ -165,7 +185,7 @@ struct ExitLocationView<ViewModel: SelectLocationViewModel>: View {
context: $viewModel.entryContext,
newCustomListAlert: nil,
alert: nil,
- onShowsTopOfTheListChange: { _ in }
+ onScrollOffsetChange: { _, _ in }
)
.background(Color.mullvadBackground)
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/Hop.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/Hop.swift
index 6411a51e67..527137f1f1 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/Hop.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/Hop.swift
@@ -5,16 +5,13 @@ struct Hop {
let selectedLocation: LocationNode?
var noMatchFound: NoMatchFoundReason? {
if let selectedLocation {
- if let customListLocation = selectedLocation as? CustomListLocationNode {
- if customListLocation.children.isEmpty {
- return .customListEmpty
- }
- }
- if !selectedLocation.isActive {
- return .selectionInactive
- } else {
- return nil
+ let userSelectedRelays = selectedLocation.userSelectedRelays
+ if userSelectedRelays.customListSelection != nil,
+ userSelectedRelays.locations.isEmpty
+ {
+ return .customListEmpty
}
+ return selectedLocation.isActive ? nil : .selectionInactive
}
return .noSelection
}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/MultihopSelectionView.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/MultihopSelectionView.swift
index d64473d8d1..c5157b3ad6 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/MultihopSelectionView.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/MultihopSelection/MultihopSelectionView.swift
@@ -161,7 +161,7 @@ struct MultihopSelectionView: View {
selectedLocation: .init(name: "\($0.description)", code: "se"))
},
selectedMultihopContext: $selectedContext,
- isExpanded: isExpanded,
+ isExpanded: isExpanded,
deviceLocationName: "Sweden"
)
.padding()
@@ -174,7 +174,7 @@ struct MultihopSelectionView: View {
)
},
selectedMultihopContext: $selectedContext,
- isExpanded: isExpanded,
+ isExpanded: isExpanded,
deviceLocationName: "Sweden"
)
.padding()
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationListItem.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationListItem.swift
new file mode 100644
index 0000000000..2cfed68aa6
--- /dev/null
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationListItem.swift
@@ -0,0 +1,32 @@
+//
+// RecentLocationListItem.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-11-27.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+
+import SwiftUI
+
+struct RecentLocationListItem<ContextMenu>: View where ContextMenu: View {
+ @Binding var location: LocationNode
+ let multihopContext: MultihopContext
+ let onSelect: (LocationNode) -> Void
+ let contextMenu: (LocationNode) -> ContextMenu
+
+ var body: some View {
+ RelayItemView(
+ location: location,
+ multihopContext: multihopContext,
+ level: 0,
+ isLastInList: true,
+ onSelect: { onSelect(location) }
+ )
+ .accessibilityIdentifier(.locationListItem(location.name))
+ .contextMenu {
+ contextMenu(location)
+ }
+ .padding(.top, 4)
+ .id(location.code) // to be able to scroll to this item programmatically
+ }
+}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationsListView.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationsListView.swift
new file mode 100644
index 0000000000..e9032d1951
--- /dev/null
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/RecentLocationsListView.swift
@@ -0,0 +1,57 @@
+//
+// RecentLocationsListView.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-11-27.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+import SwiftUI
+
+struct RecentLocationsListView<ContextMenu>: View where ContextMenu: View {
+ @Binding var locations: [LocationNode]
+ let multihopContext: MultihopContext
+ let onSelectLocation: (LocationNode) -> Void
+ let contextMenu: (LocationNode) -> ContextMenu
+
+ var filteredLocationIndices: [Int] {
+ locations
+ .enumerated()
+ .filter { !$0.element.isHiddenFromSearch }
+ .map { $0.offset }
+ }
+
+ var body: some View {
+ ForEach(
+ Array(filteredLocationIndices.enumerated()),
+ id: \.element
+ ) {
+ index,
+ indexInLocationList in
+ let location = $locations[indexInLocationList]
+ RecentLocationListItem(
+ location: location,
+ multihopContext: multihopContext,
+ onSelect: onSelectLocation,
+ contextMenu: { location in contextMenu(location) }
+ )
+ }
+ }
+}
+
+#Preview {
+ @Previewable @StateObject var viewModel = MockSelectLocationViewModel()
+ ScrollView {
+ LazyVStack(spacing: 0) {
+ RecentLocationsListView(
+ locations: $viewModel.exitContext.customLists,
+ multihopContext: .exit,
+ onSelectLocation: { location in
+ print("Selected: \(location.name)")
+ },
+ contextMenu: { location in Text("Add \(location.name) to list") }
+ )
+ .padding(.horizontal)
+ }
+ }
+ .background(Color.mullvadBackground)
+}
diff --git a/ios/MullvadVPN/View controllers/SelectLocation/Views/SelectLocationView.swift b/ios/MullvadVPN/View controllers/SelectLocation/Views/SelectLocationView.swift
index 7e25a6220f..2b7a7d3a05 100644
--- a/ios/MullvadVPN/View controllers/SelectLocation/Views/SelectLocationView.swift
+++ b/ios/MullvadVPN/View controllers/SelectLocation/Views/SelectLocationView.swift
@@ -3,14 +3,25 @@ import SwiftUI
struct SelectLocationView<ViewModel>: View where ViewModel: SelectLocationViewModel {
@ObservedObject var viewModel: ViewModel
+ @State private var headerIsExpandedForEntry: Bool = true
+ @State private var headerIsExpandedForExit: Bool = true
+ private var headerIsExpanded: Bool {
+ switch viewModel.multihopContext {
+ case .entry:
+ headerIsExpandedForEntry
+ case .exit:
+ headerIsExpandedForExit
+ }
+ }
- @State private var isAtTopOfList: Bool = false
@State private var headerHeight: CGFloat = 0
- var showSearchField: Bool {
+ private var showSearchField: Bool {
return !viewModel.showDAITAInfo || viewModel.multihopContext == .exit
}
+ @State private var disablingRecentConnectionsAlert: MullvadAlert?
+
var body: some View {
// Simply animating the MultihopSelectionView while scrolling leads to a slow
// down of the scrolling during the animation. Instead of changing the size of the scroll
@@ -27,7 +38,7 @@ struct SelectLocationView<ViewModel>: View where ViewModel: SelectLocationViewMo
)
},
selectedMultihopContext: $viewModel.multihopContext,
- isExpanded: isAtTopOfList
+ isExpanded: headerIsExpanded
)
.padding(.horizontal, 16)
if showSearchField {
@@ -58,10 +69,13 @@ struct SelectLocationView<ViewModel>: View where ViewModel: SelectLocationViewMo
ExitLocationView(
viewModel: viewModel,
context: $viewModel.exitContext,
- onShowsTopOfTheListChange: { onTop in
- withAnimation {
- isAtTopOfList = onTop
- }
+ onScrollOffsetChange: {
+ prevScrollOffset,
+ scrollOffset in
+ expandOrCollapseHeader(
+ prevScrollOffset: prevScrollOffset,
+ scrollOffset: scrollOffset,
+ context: .exit)
}
)
.transition(
@@ -70,10 +84,11 @@ struct SelectLocationView<ViewModel>: View where ViewModel: SelectLocationViewMo
case .entry:
EntryLocationView(
viewModel: viewModel,
- onShowsTopOfTheListChange: { onTop in
- withAnimation {
- isAtTopOfList = onTop
- }
+ onScrollOffsetChange: { prevScrollOffset, scrollOffset in
+ expandOrCollapseHeader(
+ prevScrollOffset: prevScrollOffset,
+ scrollOffset: scrollOffset,
+ context: .entry)
}
)
.transition(
@@ -118,6 +133,34 @@ struct SelectLocationView<ViewModel>: View where ViewModel: SelectLocationViewMo
.foregroundStyle(Color.mullvadTextPrimary)
}
.accessibilityIdentifier(.selectLocationFilterButton)
+
+ Button {
+ if viewModel.isRecentsEnabled {
+ disablingRecentConnectionsAlert = MullvadAlert(
+ type: .warning,
+ messages: ["Disabling recents will also clear history."],
+ action: MullvadAlert.Action(
+ type: .danger,
+ title: "Disable",
+ identifier: .disableRecentConnectionsButton,
+ handler: {
+ viewModel.toggleRecents()
+ disablingRecentConnectionsAlert = nil
+ }), dismissButtonTitle: "Cancel")
+
+ } else {
+ viewModel.toggleRecents()
+ }
+
+ } label: {
+ HStack {
+ Image(systemName: "clock")
+ Text("\(viewModel.isRecentsEnabled ? "Disable" : "Enable") recents")
+ }
+ .foregroundStyle(Color.mullvadTextPrimary)
+ }
+ .accessibilityIdentifier(.recentConnectionsToggleButton)
+
} label: {
Image(systemName: "ellipsis.circle.fill")
.foregroundStyle(Color.mullvadTextPrimary)
@@ -126,6 +169,46 @@ struct SelectLocationView<ViewModel>: View where ViewModel: SelectLocationViewMo
}
)
}
+ .mullvadAlert(item: $disablingRecentConnectionsAlert)
+ }
+
+ // Expands when the scroll view is at its top.
+ // Colappses if scroll view scrolls down beyond a certain point.
+ // The dead zone needs to be bigger than the height difference between collapsed and expanded state to avoid false triggering due to the UI frame sizes jumping on collapse/expand
+ private func expandOrCollapseHeader(
+ prevScrollOffset: CGFloat,
+ scrollOffset: CGFloat,
+ context: MultihopContext
+ ) {
+ let isScrollingDown = prevScrollOffset > scrollOffset
+
+ let correctedOffset = abs(min((scrollOffset - headerHeight + 1), 0))
+ if headerIsExpanded && isScrollingDown {
+ if correctedOffset > headerHeight {
+ withAnimation {
+ switch context {
+ case .entry:
+ headerIsExpandedForEntry = false
+ case .exit:
+ headerIsExpandedForExit = false
+ }
+ }
+ return
+ }
+ }
+ if !headerIsExpanded && !isScrollingDown {
+ if correctedOffset == 0 {
+ withAnimation {
+ switch context {
+ case .entry:
+ headerIsExpandedForEntry = true
+ case .exit:
+ headerIsExpandedForExit = true
+ }
+ }
+ return
+ }
+ }
}
}
diff --git a/ios/MullvadVPN/Views/MullvadListSectionFooter.swift b/ios/MullvadVPN/Views/MullvadListSectionFooter.swift
new file mode 100644
index 0000000000..aa1bbf38fa
--- /dev/null
+++ b/ios/MullvadVPN/Views/MullvadListSectionFooter.swift
@@ -0,0 +1,23 @@
+//
+// MullvadListSectionFooter.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-11-25.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+
+import SwiftUI
+
+struct MullvadListSectionFooter: View {
+ let title: LocalizedStringKey
+ var body: some View {
+ Text(title)
+ .font(.mullvadMini)
+ .foregroundStyle(Color.mullvadTextPrimary.opacity(0.6))
+ .padding(.bottom, 24)
+ .frame(maxWidth: .infinity, alignment: .leading)
+ }
+}
+#Preview {
+ MullvadListSectionFooter(title: "Custom lists")
+}
diff --git a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/AllLocationsDataSourceTests.swift b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/AllLocationsDataSourceTests.swift
index fcfd6af20b..b0c09b8a88 100644
--- a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/AllLocationsDataSourceTests.swift
+++ b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/AllLocationsDataSourceTests.swift
@@ -96,90 +96,57 @@ class AllLocationsDataSourceTests: XCTestCase {
XCTAssertEqual(nodeByLocation, nodeByCode)
}
- func testConnectedNode() throws {
- let hostname = "es1-wireguard"
- dataSource.setConnectedRelay(hostname: hostname)
- dataSource.nodes.forEachNode { node in
- XCTAssertEqual(node.isConnected, node.name == hostname)
- }
-
- dataSource.setConnectedRelay(hostname: "invalid-hostname")
- dataSource.nodes.forEachNode { node in
- XCTAssertFalse(node.isConnected)
- }
- }
-
- func testSetSelectedLocation() throws {
- dataSource.setSelectedNode(selectedRelays: .init(locations: [.country("es")]))
-
- dataSource.nodes.forEachNode { node in
- if node.locations == [.country("es")] {
- XCTAssertTrue(node.isSelected)
- } else {
- XCTAssertFalse(node.isSelected)
- }
- }
-
- dataSource
- .setSelectedNode(
- selectedRelays: .init(locations: [.country("invalid")])
- )
- dataSource.nodes.forEachNode { node in
- XCTAssertFalse(node.isSelected)
- }
- }
-
- func testDoNotSetSelectedCustomListLocation() throws {
- let selectedRelays: UserSelectedRelays = .init(
- locations: [
- .country("es")
- ],
- customListSelection: .init(listId: .init(), isList: false)
- )
-
- dataSource.setSelectedNode(selectedRelays: selectedRelays)
-
- dataSource.nodes.forEachNode { node in
- XCTAssertFalse(node.isSelected)
- }
- }
-
- func testExcludeLocation() throws {
- let excludedRelays = UserSelectedRelays(locations: [.hostname("se", "sto", "se2-wireguard")])
- dataSource.setExcludedNode(excludedSelection: excludedRelays)
- let excludedNode = dataSource.node(by: excludedRelays)!
-
- XCTAssertTrue(excludedNode.isExcluded)
-
- excludedNode.forEachAncestor { ancestor in
- XCTAssertFalse(ancestor.isExcluded)
- }
-
- let includedNode = dataSource.node(by: .init(locations: [.country("es")]))!
- XCTAssertFalse(includedNode.isExcluded)
- includedNode.forEachDescendant { child in
- XCTAssertFalse(child.isExcluded)
- }
- }
-
- func testExcludeLocationIncludesAncestors() throws {
- let excludedRelays = UserSelectedRelays(locations: [.hostname("es", "mad", "es1-wireguard")])
- dataSource.setExcludedNode(excludedSelection: excludedRelays)
- let excludedNode = dataSource.node(by: excludedRelays)!
-
- XCTAssertTrue(excludedNode.isExcluded)
-
- // All ancestors are exluded when single child is excluded
- excludedNode.forEachAncestor { ancestor in
- XCTAssertTrue(ancestor.isExcluded)
- }
-
- let includedNode = dataSource.node(by: .init(locations: [.country("se")]))!
- XCTAssertFalse(includedNode.isExcluded)
- includedNode.forEachDescendant { child in
- XCTAssertFalse(child.isExcluded)
- }
- }
+ // func testConnectedNode() throws {
+ // let hostname = "es1-wireguard"
+ // dataSource.setConnectedRelay(hostname: hostname)
+ // dataSource.nodes.forEachNode { node in
+ // XCTAssertEqual(node.isConnected, node.name == hostname)
+ // }
+ //
+ // dataSource.setConnectedRelay(hostname: "invalid-hostname")
+ // dataSource.nodes.forEachNode { node in
+ // XCTAssertFalse(node.isConnected)
+ // }
+ // }
+ //
+ // func testExcludeLocation() throws {
+ // let excludedRelays = UserSelectedRelays(locations: [.hostname("se", "sto", "se2-wireguard")])
+ // dataSource.setExcludedNode(excludedSelection: excludedRelays)
+ // let excludedNode = dataSource.node(by: excludedRelays)!
+ //
+ // XCTAssertTrue(excludedNode.isExcluded)
+ //
+ // excludedNode.forEachAncestor { ancestor in
+ // XCTAssertFalse(ancestor.isExcluded)
+ // }
+ //
+ // let includedRelays = UserSelectedRelays(locations: [.country("es")])
+ // let includedNode = dataSource.node(by: includedRelays)!
+ // XCTAssertFalse(includedNode.isExcluded)
+ // includedNode.forEachDescendant { child in
+ // XCTAssertFalse(child.isExcluded)
+ // }
+ // }
+ //
+ // func testExcludeLocationIncludesAncestors() throws {
+ // let excludedRelays = UserSelectedRelays(locations: [.hostname("es", "mad", "es1-wireguard")])
+ // dataSource.setExcludedNode(excludedSelection: excludedRelays)
+ // let excludedNode = dataSource.node(by: excludedRelays)!
+ //
+ // XCTAssertTrue(excludedNode.isExcluded)
+ //
+ // // All ancestors are exluded when single child is excluded
+ // excludedNode.forEachAncestor { ancestor in
+ // XCTAssertTrue(ancestor.isExcluded)
+ // }
+ //
+ // let includedRelays = UserSelectedRelays(locations: [.country("se")])
+ // let includedNode = dataSource.node(by: includedRelays)!
+ // XCTAssertFalse(includedNode.isExcluded)
+ // includedNode.forEachDescendant { child in
+ // XCTAssertFalse(child.isExcluded)
+ // }
+ // }
}
extension AllLocationsDataSourceTests {
diff --git a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/CustomListsDataSourceTests.swift b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/CustomListsDataSourceTests.swift
index e9946d8c7b..5301417bfe 100644
--- a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/CustomListsDataSourceTests.swift
+++ b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/CustomListsDataSourceTests.swift
@@ -83,35 +83,6 @@ class CustomListsDataSourceTests: XCTestCase {
XCTAssertEqual(nodeByLocations, nodeByCode)
}
- func testSetSelection() throws {
- let customListId = (dataSource.nodes.first! as! CustomListLocationNode).customList.id
- let userSelectedRelays = UserSelectedRelays(
- locations: [.country("se")],
- customListSelection: .init(listId: customListId, isList: false)
- )
-
- dataSource
- .setSelectedNode(
- selectedRelays: userSelectedRelays
- )
-
- dataSource.nodes.forEachNode { node in
- if node.locations == [.country("se")] {
- XCTAssertTrue(node.isSelected)
- } else {
- XCTAssertFalse(node.isSelected)
- }
- }
-
- dataSource
- .setSelectedNode(
- selectedRelays: .init(locations: [.country("invalid")])
- )
- dataSource.nodes.forEachNode { node in
- XCTAssertFalse(node.isSelected)
- }
- }
-
func testDoNotSetSelectedLocation() throws {
let selectedRelays: UserSelectedRelays = .init(
locations: [
@@ -119,7 +90,8 @@ class CustomListsDataSourceTests: XCTestCase {
]
)
- dataSource.setSelectedNode(selectedRelays: selectedRelays)
+ let nodeByLocations = dataSource.node(by: selectedRelays)
+ nodeByLocations?.isSelected = true
dataSource.nodes.forEachNode { node in
XCTAssertFalse(node.isSelected)
diff --git a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/LocationNodeResolverTests.swift b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/LocationNodeResolverTests.swift
new file mode 100644
index 0000000000..6543f3022d
--- /dev/null
+++ b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/LocationNodeResolverTests.swift
@@ -0,0 +1,75 @@
+//
+// LocationNodeResolverTests.swift
+// MullvadVPN
+//
+// Created by Mojgan on 2025-12-04.
+// Copyright © 2025 Mullvad VPN AB. All rights reserved.
+//
+
+import MullvadMockData
+import Testing
+
+@testable import MullvadSettings
+@testable import MullvadTypes
+
+struct LocationNodeResolverTests {
+ let customListDataSource: CustomListsDataSource
+ let allLocationDataSource: AllLocationDataSource
+ let recentListDataSource: RecentListDataSource
+ let resolver: LocationNodeResolver
+
+ init() {
+ let response = ServerRelaysResponseStubs.sampleRelays
+ let relays = LocationRelays(relays: response.wireguard.relays, locations: response.locations)
+
+ allLocationDataSource = AllLocationDataSource()
+ customListDataSource = CustomListsDataSource(
+ repository: CustomListsRepositoryStub(customLists: Self.customLists))
+
+ allLocationDataSource.reload(relays)
+ customListDataSource.reload(allLocationNodes: allLocationDataSource.nodes)
+
+ recentListDataSource = RecentListDataSource(allLocationDataSource, customListsDataSource: customListDataSource)
+ recentListDataSource.reload(Self.recents)
+
+ resolver = LocationNodeResolver(providers: [recentListDataSource, customListDataSource, allLocationDataSource])
+ }
+
+ @Test
+ func testSingleSelection() {
+ resolver.setSelectedNode(selectedRelays: Self.recents.first!)
+ let allNodes = allLocationDataSource.nodes + customListDataSource.nodes + recentListDataSource.nodes
+ let count = allNodes.count(where: { $0.flattened.contains(where: { $0.isSelected }) })
+ #expect(allNodes.count(where: { $0.flattened.contains(where: { $0.isSelected }) }) == 1)
+ }
+}
+
+extension LocationNodeResolverTests {
+ private static var customLists: [CustomList] {
+ [
+ CustomList(
+ name: "Netflix",
+ locations: [
+ .hostname("es", "mad", "es1-wireguard"),
+ .country("se"),
+ .city("us", "dal"),
+ ]),
+ CustomList(
+ name: "Youtube",
+ locations: [
+ .hostname("se", "sto", "se2-wireguard"),
+ .city("us", "dal"),
+ ]),
+ ]
+ }
+
+ private static var recents: [UserSelectedRelays] {
+ [
+ UserSelectedRelays(locations: [.country("se")]),
+ UserSelectedRelays(
+ locations: customLists.first!.locations,
+ customListSelection: UserSelectedRelays.CustomListSelection(listId: customLists.first!.id, isList: true)
+ ),
+ ]
+ }
+}
diff --git a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/RecentConnectionsRepositoryTests.swift b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/RecentConnectionsRepositoryTests.swift
index 2d63e64563..1de46194b4 100644
--- a/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/RecentConnectionsRepositoryTests.swift
+++ b/ios/MullvadVPNTests/MullvadVPN/View controllers/SelectLocation/RecentConnectionsRepositoryTests.swift
@@ -5,6 +5,8 @@
// Created by Mojgan on 2025-09-25.
// Copyright © 2025 Mullvad VPN AB. All rights reserved.
//
+
+import Combine
import Testing
@testable import MullvadSettings
@@ -16,80 +18,121 @@ final class RecentConnectionsRepositoryTests {
let fr = UserSelectedRelays(locations: [.country("fr")])
let nl = UserSelectedRelays(locations: [.country("nl")])
let de = UserSelectedRelays(locations: [.country("de")])
+ private var cancellables = Set<Combine.AnyCancellable>()
@Test("Adds locations up to the limit 1 for either entry or exit")
func addLocations() throws {
let maxLimit: UInt = 1
let repository = makeRepository(max: maxLimit)
- try repository.setRecentsEnabled(true)
- try addLocations(repository, locations: [se, de], as: .entry)
- try addLocations(repository, locations: [de], as: .exit)
+ var recentConnections: RecentConnections?
+ var thrownError: Error?
+
+ repository
+ .recentConnectionsPublisher
+ .sink(receiveValue: { result in
+ switch result {
+ case .success(let value):
+ recentConnections = value
+ case .failure(let error):
+ thrownError = error
+ }
+ })
+ .store(in: &cancellables)
- let recentsSettings = try repository.all()
- #expect(recentsSettings.isEnabled)
- #expect(recentsSettings.exitLocations.count == maxLimit)
- #expect(recentsSettings.entryLocations.count == maxLimit)
+ repository.enable(se, selectedExitRelays: de)
+ repository.add(de, selectedExitRelays: se)
+
+ let value = try #require(recentConnections)
+ #expect(thrownError == nil)
+ #expect(value.isEnabled)
+ #expect(value.exitLocations.count == maxLimit)
+ #expect(value.entryLocations.count == maxLimit)
}
@Test("Adds locations up to the default limit (50) for either entry or exit")
func addDuplicate() throws {
let repository = makeRepository()
- try repository.setRecentsEnabled(true)
- try addLocations(repository, locations: [se, de], as: .entry)
- try addLocations(repository, locations: [de, se, nl, se], as: .exit)
+ var recentConnections: RecentConnections?
+ var thrownError: Error?
+
+ repository
+ .recentConnectionsPublisher
+ .sink(receiveValue: { result in
+ switch result {
+ case .success(let value):
+ recentConnections = value
+ case .failure(let error):
+ thrownError = error
+ }
+ })
+ .store(in: &cancellables)
- let recentsSettings = try repository.all()
- #expect(recentsSettings.isEnabled)
- #expect(recentsSettings.entryLocations.count == 2)
- #expect(recentsSettings.exitLocations.count == 3)
+ repository.enable(se, selectedExitRelays: de)
+ repository.add(de, selectedExitRelays: se)
+ repository.add(de, selectedExitRelays: nl)
+
+ let value = try #require(recentConnections)
+ #expect(thrownError == nil)
+ #expect(value.isEnabled)
+ #expect(value.entryLocations.count == 2)
+ #expect(value.exitLocations.count == 3)
}
@Test("Removes all recents connections with disabling recents")
func disable() throws {
let repository = makeRepository()
- try repository.setRecentsEnabled(true)
- try addLocations(repository, locations: [se, de], as: .entry)
- try addLocations(repository, locations: [de, se, nl], as: .exit)
- var recentConnections = try repository.all()
- #expect(recentConnections.isEnabled)
- #expect(recentConnections.entryLocations.count == 2)
- #expect(recentConnections.exitLocations.count == 3)
+ var recentConnections: RecentConnections?
+ var thrownError: Error?
+
+ repository
+ .recentConnectionsPublisher
+ .sink(receiveValue: { result in
+ switch result {
+ case .success(let value):
+ recentConnections = value
+ case .failure(let error):
+ thrownError = error
+ }
+ })
+ .store(in: &cancellables)
- try repository.setRecentsEnabled(false)
+ repository.disable()
- recentConnections = try repository.all()
- #expect(!recentConnections.isEnabled)
- #expect(recentConnections.entryLocations.count == 0)
- #expect(recentConnections.exitLocations.count == 0)
+ let value = try #require(recentConnections)
+ #expect(thrownError == nil)
+ #expect(!value.isEnabled)
+ #expect(value.entryLocations.count == 0)
+ #expect(value.exitLocations.count == 0)
}
@Test("Fails with an error if a location is added while recents are disabled.")
func addRecentsBeforeEnablingRecents() throws {
let repository = makeRepository()
+ repository.disable()
- try repository.setRecentsEnabled(false)
- let action: () throws -> Void = { [self] in
- try addLocations(
- repository,
- locations: [self.se],
- as: RecentLocationType.entry
- )
- }
+ var recentConnections: RecentConnections?
+ var thrownError: Error?
+ repository
+ .recentConnectionsPublisher
+ .sink(receiveValue: { result in
+ switch result {
+ case .success(let value):
+ recentConnections = value
+ case .failure(let error):
+ thrownError = error
+ }
+ })
+ .store(in: &cancellables)
+ repository.add(nil, selectedExitRelays: se)
- #expect(throws: RecentConnectionsRepositoryError.recentsDisabled, performing: action)
+ let error = try #require(thrownError as? RecentConnectionsRepositoryError)
+ #expect(error == RecentConnectionsRepositoryError.recentsDisabled)
+ #expect(recentConnections == nil)
}
private func makeRepository(max: UInt = 50) -> RecentConnectionsRepository {
return RecentConnectionsRepository(store: InMemorySettingsStore<SettingNotFound>(), maxLimit: max)
}
-
- private func addLocations(
- _ repository: RecentConnectionsRepository, locations: [UserSelectedRelays], as type: RecentLocationType
- ) throws {
- for location in locations {
- try repository.add(location, as: type)
- }
- }
}
diff --git a/ios/MullvadSettings/RecentConnectionsRepository.swift b/ios/MullvadSettings/RecentConnectionsRepository.swift
index b655fd6156..396d916364 100644
--- a/ios/MullvadSettings/RecentConnectionsRepository.swift
+++ b/ios/MullvadSettings/RecentConnectionsRepository.swift
@@ -5,6 +5,8 @@
// Created by Mojgan on 2025-10-15.
// Copyright © 2025 Mullvad VPN AB. All rights reserved.
//
+
+import Combine
import MullvadTypes
public enum RecentConnectionsRepositoryError: LocalizedError, Hashable {
@@ -18,55 +20,94 @@ public enum RecentConnectionsRepositoryError: LocalizedError, Hashable {
}
}
-final class RecentConnectionsRepository: RecentConnectionsRepositoryProtocol {
+final public class RecentConnectionsRepository: RecentConnectionsRepositoryProtocol {
private let store: SettingsStore
private let maxLimit: UInt
+ private let recentConnectionsSubject: PassthroughSubject<RecentConnectionsResult, Never> = .init()
private let settingsParser: SettingsParser = {
SettingsParser(decoder: JSONDecoder(), encoder: JSONEncoder())
}()
- init(store: SettingsStore, maxLimit: UInt = 50) {
+ public var recentConnectionsPublisher: AnyPublisher<RecentConnectionsResult, Never> {
+ recentConnectionsSubject.eraseToAnyPublisher()
+ }
+
+ public init(store: SettingsStore, maxLimit: UInt = 50) {
self.store = store
self.maxLimit = maxLimit
}
- func setRecentsEnabled(_ isEnabled: Bool) throws {
- // Clear all recents whenever the recents feature status changes.
- try write(RecentConnections(isEnabled: isEnabled, entryLocations: [], exitLocations: []))
+ public func disable() {
+ do {
+ // Clear all recents whenever the recents feature status changes.
+ let value = RecentConnections(isEnabled: false, entryLocations: [], exitLocations: [])
+ try write(value)
+ recentConnectionsSubject.send(.success(value))
+ } catch {
+ recentConnectionsSubject.send(.failure(error))
+ }
+ }
+ public func enable(_ selectedEntryRelays: UserSelectedRelays?, selectedExitRelays: UserSelectedRelays) {
+ do {
+ // Enable recents with the last selected locations for entry and exit.
+ let value = RecentConnections(
+ entryLocations: (selectedEntryRelays != nil) ? [selectedEntryRelays!] : [],
+ exitLocations: [selectedExitRelays])
+ try write(value)
+ recentConnectionsSubject.send(.success(value))
+ } catch {
+ recentConnectionsSubject.send(.failure(error))
+ }
}
- func add(_ location: UserSelectedRelays, as type: RecentLocationType) throws {
- let current = try read()
- guard current.isEnabled else { throw RecentConnectionsRepositoryError.recentsDisabled }
- var currentList = current[keyPath: keyPath(for: type)]
- if let idx = currentList.firstIndex(of: location) { currentList.remove(at: idx) }
- currentList.insert(location, at: 0)
- currentList = Array(currentList.prefix(Int(maxLimit)))
+ public func add(_ selectedEntryRelays: UserSelectedRelays?, selectedExitRelays: UserSelectedRelays) {
+ do {
+ let current = try read()
+ guard current.isEnabled else { throw RecentConnectionsRepositoryError.recentsDisabled }
- let new =
- (type == .entry)
- ? RecentConnections(
- isEnabled: current.isEnabled, entryLocations: currentList, exitLocations: current.exitLocations)
- : RecentConnections(
- isEnabled: current.isEnabled, entryLocations: current.entryLocations, exitLocations: currentList)
+ let insertAtZero: ([UserSelectedRelays], UserSelectedRelays?) -> [UserSelectedRelays] = {
+ (locations, location) in
+ guard let location = location else { return locations }
+ var currentLocations = locations
+ currentLocations.removeAll(where: {
+ // If the item represents the same custom list, remove it so the list
+ // can be refreshed with the updated value. Otherwise, remove it only
+ // if it matches the same location to avoid duplicate recent locations.
+ if let customList = $0.customListSelection,
+ customList == location.customListSelection
+ {
+ return true
+ } else {
+ return $0.locations == location.locations
+ }
+ })
+ currentLocations.insert(location, at: 0)
+ return Array(currentLocations.prefix(Int(self.maxLimit)))
+ }
- try write(new)
- }
+ let new = RecentConnections(
+ entryLocations: insertAtZero(current.entryLocations, selectedEntryRelays),
+ exitLocations: insertAtZero(current.exitLocations, selectedExitRelays))
+ try write(new)
+ recentConnectionsSubject.send(.success(new))
- func all() throws -> RecentConnections {
- try read()
+ } catch {
+ recentConnectionsSubject.send(.failure(error))
+ }
}
-}
-private extension RecentConnectionsRepository {
- private func keyPath(for type: RecentLocationType) -> KeyPath<RecentConnections, [UserSelectedRelays]> {
- switch type {
- case .entry: return \.entryLocations
- case .exit: return \.exitLocations
+ public func initiate() {
+ do {
+ let value = try read()
+ recentConnectionsSubject.send(.success(value))
+ } catch {
+ recentConnectionsSubject.send(.failure(error))
}
}
+}
+private extension RecentConnectionsRepository {
private func read() throws -> RecentConnections {
let data = try store.read(key: .recentConnections)
return try settingsParser.parseUnversionedPayload(as: RecentConnections.self, from: data)